# -*- coding: utf-8 -*-
from __future__ import print_function


def partition_integer_into_parts_less_than_double(n, m):
    r'''Partitions integer `n` into parts less than double integer `m`.

    ..  container:: example

        ::

            >>> for n in range(1, 24+1):
            ...     print(n, mathtools.partition_integer_into_parts_less_than_double(n, 4))
            1 (1,)
            2 (2,)
            3 (3,)
            4 (4,)
            5 (5,)
            6 (6,)
            7 (7,)
            8 (4, 4)
            9 (4, 5)
            10 (4, 6)
            11 (4, 7)
            12 (4, 4, 4)
            13 (4, 4, 5)
            14 (4, 4, 6)
            15 (4, 4, 7)
            16 (4, 4, 4, 4)
            17 (4, 4, 4, 5)
            18 (4, 4, 4, 6)
            19 (4, 4, 4, 7)
            20 (4, 4, 4, 4, 4)
            21 (4, 4, 4, 4, 5)
            22 (4, 4, 4, 4, 6)
            23 (4, 4, 4, 4, 7)
            24 (4, 4, 4, 4, 4, 4)

    Returns tuple of one or more integers.
    '''
    from abjad.tools import mathtools
    assert mathtools.is_positive_integer_equivalent_number(n)
    assert mathtools.is_positive_integer_equivalent_number(m)
    n, m = int(n), int(m)
    result = []
    current_value = n
    double_m = 2 * m
    while double_m <= current_value:
        result.append(m)
        current_value -= m
    result.append(current_value)
    return tuple(result)



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_leaf_pair_01():

    score = Score([])
    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8"), Note("g'4")]
    score.append(Staff(notes))
    notes = [Note(x, (1, 4)) for x in [-12, -15, -17]]
    score.append(Staff(notes))
    clef = Clef('bass')
    attach(clef, score[1])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
                g'4
            }
            \new Staff {
                \clef "bass"
                c4
                a,4
                g,4
            }
        >>
        '''
        )

    # make sure to render score PDF if trying to verify test results
    # as the integer indices below are difficult to keep in mind.

    pairs = iterate(score).by_leaf_pair()
    pairs = list(pairs)

    assert len(pairs) == 15
    assert pairs[0] == (score[0][0], score[1][0])
    assert pairs[1] == (score[0][0], score[0][1])
    assert pairs[2] == (score[1][0], score[0][1])
    assert pairs[3] == (score[0][1], score[0][2])
    assert pairs[4] == (score[0][1], score[1][1])
    assert pairs[5] == (score[1][0], score[0][2])
    assert pairs[6] == (score[1][0], score[1][1])
    assert pairs[7] == (score[0][2], score[1][1])
    assert pairs[8] == (score[0][2], score[0][3])
    assert pairs[9] == (score[1][1], score[0][3])
    assert pairs[10] == (score[0][3], score[0][4])
    assert pairs[11] == (score[0][3], score[1][2])
    assert pairs[12] == (score[1][1], score[0][4])
    assert pairs[13] == (score[1][1], score[1][2])
    assert pairs[14] == (score[0][4], score[1][2])



# -*- coding: utf-8 -*-
from sphinx.util.compat import Directive


class DoctestDirective(Directive):
    r'''An abjad-book doctest directive.

    Contributes no formatting to documents built by Sphinx.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Sphinx Internals'

    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    ### PUBLIC METHODS ###

    def run(self):
        r'''Executes the directive.
        '''
        self.assert_has_content()
        return []



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.QEvent import QEvent


class TerminalQEvent(QEvent):
    r'''The terminal event in a series of ``QEvents``:

        >>> q_event = quantizationtools.TerminalQEvent(1000)
        >>> print(format(q_event))
        quantizationtools.TerminalQEvent(
            offset=durationtools.Offset(1000, 1),
            )

    Carries no significance outside the context of a ``QEventSequence``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_offset',
        )

    ### INITIALIZER ###

    def __init__(self, offset=0):
        QEvent.__init__(self, offset=offset)

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a terminal q-event with offset equal to that
        of this terminal q-event. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument) and \
            self.offset == argument.offset:
            return True
        return False

    def __hash__(self):
        r'''Hashes terminal q-event.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(TerminalQEvent, self).__hash__()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_piano_sketch_staff_from_leaves_01():

    notes = scoretools.make_notes([-12, -10, -8, -7, -5, 0, 2, 4, 5, 7], [(1, 4)])
    score, treble_staff, bass_staff = scoretools.make_piano_sketch_score_from_leaves(notes)

    r'''
    \new Score \with {
        \override BarLine.stencil = ##f
        \override BarNumber.transparent = ##t
        \override SpanBar.stencil = ##f
        \override TimeSignature.stencil = ##f
    } <<
        \new PianoStaff <<
            \context Staff = "treble" {
                \clef "treble"
                r4
                r4
                r4
                r4
                r4
                c'4
                d'4
                e'4
                f'4
                g'4
            }
            \context Staff = "bass" {
                \clef "bass"
                c4
                d4
                e4
                f4
                g4
                r4
                r4
                r4
                r4
                r4
            }
        >>
    >>
    '''

    assert format(score) == stringtools.normalize(
        r'''
        \new Score \with {
            \override BarLine.stencil = ##f
            \override BarNumber.transparent = ##t
            \override SpanBar.stencil = ##f
            \override TimeSignature.stencil = ##f
        } <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                    r4
                    r4
                    r4
                    r4
                    r4
                    c'4
                    d'4
                    e'4
                    f'4
                    g'4
                }
                \context Staff = "bass" {
                    \clef "bass"
                    c4
                    d4
                    e4
                    f4
                    g4
                    r4
                    r4
                    r4
                    r4
                    r4
                }
            >>
        >>
        '''
        )



# -*- coding: utf-8 -*-
from __future__ import print_function
import abc
import bisect
import copy
from abjad.tools import durationtools
from abjad.tools import systemtools
from abjad.tools import mathtools
from abjad.tools import selectiontools
from abjad.tools import timespantools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_
from abjad.tools.abctools import AbjadObject


class Component(AbjadObject):
    r'''Abstract base class from which score components inherit.

    Notes, rests, chords, tuplets, measures, voices, staves, staff groups and
    scores are all components.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_after_grace',
        '_dependent_expressions',
        '_grace',
        '_indicator_expressions',
        '_indicators_are_current',
        '_is_forbidden_to_update',
        '_lilypond_grob_name_manager',
        '_lilypond_setting_name_manager',
        '_logical_measure_number',
        '_name',
        '_offsets_are_current',
        '_offsets_in_seconds_are_current',
        '_parent',
        '_spanners',
        '_start_offset',
        '_start_offset_in_seconds',
        '_stop_offset',
        '_stop_offset_in_seconds',
        '_timespan',
        )

    _is_counttime_component = False

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, name=None):
        self._after_grace = None
        self._dependent_expressions = []
        self._grace = None
        self._indicator_expressions = []
        self._indicators_are_current = False
        self._is_forbidden_to_update = False
        self._logical_measure_number = None
        self._offsets_are_current = False
        self._offsets_in_seconds_are_current = False
        self._lilypond_grob_name_manager = None
        self._parent = None
        self._lilypond_setting_name_manager = None
        self._spanners = set()
        self._start_offset = None
        self._start_offset_in_seconds = None
        self._stop_offset = None
        self._stop_offset_in_seconds = None
        self._timespan = timespantools.Timespan()
        self._name = None
        if name is not None:
            self.name = name  # name must be setup *after* parent

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        r'''Copies component with indicators but without children of component
        or spanners attached to component.

        Returns new component.
        '''
        return self._copy_with_indicators_but_without_children_or_spanners()

    def __format__(self, format_specification=''):
        r'''Formats component.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return ()

    def __illustrate__(self):
        r'''Illustrates component.

        Returns LilyPond file.
        '''
        import abjad
        lilypond_file = abjad.LilyPondFile.new(self)
        lilypond_file.header_block.tagline = False
        return lilypond_file

    def __mul__(self, n):
        r'''Copies component `n` times and detaches spanners.

        Returns list of new components.
        '''
        from abjad.tools import spannertools
        result = mutate(self).copy(n=n)
        for component in iterate(result).by_class():
            detach(spannertools.Spanner, component)
        if isinstance(result, type(self)):
            result = [result]
        else:
            result = list(result)
        result = selectiontools.Selection(result)
        return result

    def __repr__(self):
        '''Gets interpreter representation of leaf.

        Returns string.
        '''
        return systemtools.StorageFormatAgent(self).get_repr_format()

    def __rmul__(self, n):
        r'''Copies component `n` times and detach spanners.

        Returns list of new components.
        '''
        return self * n

    ### PRIVATE METHODS ###

    def _as_graphviz_node(self):
        from abjad.tools import graphtools
        score_index = self._get_parentage().score_index
        score_index = '_'.join(str(_) for _ in score_index)
        class_name = type(self).__name__
        if score_index:
            name = '{}_{}'.format(class_name, score_index)
        else:
            name = class_name
        node = graphtools.GraphvizNode(
            name=name,
            attributes={
                'margin': 0.05,
                },
            )
        table = graphtools.GraphvizTable(
            attributes={
                'border': 2,
                'cellpadding': 5,
                'style': 'rounded',
                },
            )
        node.append(table)
        return node

    def _cache_named_children(self):
        name_dictionary = {}
        if hasattr(self, '_named_children'):
            for name, children in self._named_children.items():
                name_dictionary[name] = copy.copy(children)
        if hasattr(self, 'name') and self.name is not None:
            if self.name not in name_dictionary:
                name_dictionary[self.name] = []
            name_dictionary[self.name].append(self)
        return name_dictionary

    def _check_for_cycles(self, components):
        parentage = self._get_parentage()
        for component in components:
            if component in parentage:
                return True
        return False

    def _copy_with_children_and_indicators_but_without_spanners(self):
        return self._copy_with_indicators_but_without_children_or_spanners()

    def _copy_with_indicators_but_without_children_or_spanners(self):
        new = type(self)(*self.__getnewargs__())
        if getattr(self, '_lilypond_grob_name_manager', None) is not None:
            new._lilypond_grob_name_manager = copy.copy(override(self))
        if getattr(self, '_lilypond_setting_name_manager', None) is not None:
            new._lilypond_setting_name_manager = copy.copy(set_(self))
        for indicator in self._get_indicators(unwrap=False):
            new_indicator = copy.copy(indicator)
            attach(new_indicator, new)
        return new

    def _detach_grace_containers(self, kind=None):
        grace_containers = self._get_grace_containers(kind=kind)
        for grace_container in grace_containers:
            detach(grace_container, self)
        return grace_containers

    def _detach_spanners(self, prototype=None):
        spanners = self._get_spanners(prototype=prototype)
        for spanner in spanners:
            spanner._sever_all_components()
        return spanners

    def _extract(self, scale_contents=False):
        from abjad.tools import selectiontools
        if scale_contents:
            self._scale_contents(self.multiplier)
        selection = selectiontools.Selection([self])
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        music_list = list(getattr(self, '_music', ()))
        parent.__setitem__(slice(start, stop + 1), music_list)
        return self

    def _format_after_slot(self, bundle):
        pass

    def _format_before_slot(self, bundle):
        pass

    def _format_close_brackets_slot(self, bundle):
        pass

    def _format_closing_slot(self, bundle):
        pass

    def _format_component(self, pieces=False):
        result = []
        manager = systemtools.LilyPondFormatManager
        bundle = manager.bundle_format_contributions(self)
        result.extend(self._format_before_slot(bundle))
        result.extend(self._format_open_brackets_slot(bundle))
        result.extend(self._format_opening_slot(bundle))
        result.extend(self._format_contents_slot(bundle))
        result.extend(self._format_closing_slot(bundle))
        result.extend(self._format_close_brackets_slot(bundle))
        result.extend(self._format_after_slot(bundle))
        contributions = []
        for contributor, contribution in result:
            contributions.extend(contribution)
        if pieces:
            return contributions
        else:
            return '\n'.join(contributions)

    def _format_contents_slot(self, bundle):
        pass

    def _format_open_brackets_slot(self, bundle):
        pass

    def _format_opening_slot(self, bundle):
        pass

    def _get_components(self, prototype=None, include_self=True):
        argument = self
        if include_self:
            argument = [self]
        components = iterate(argument).by_class(prototype)
        return selectiontools.Selection(components)

    def _get_contents(self, include_self=True):
        result = []
        if include_self:
            result.append(self)
        result.extend(getattr(self, '_music', []))
        result = selectiontools.Selection(result)
        return result

    def _get_descendants(
        self,
        include_self=True,
        ):
        return selectiontools.Descendants(
            self,
            include_self=include_self,
            )

    def _get_descendants_starting_with(self):
        from abjad.tools import scoretools
        result = []
        result.append(self)
        if isinstance(self, scoretools.Container):
            if self.is_simultaneous:
                for x in self:
                    result.extend(x._get_descendants_starting_with())
            elif self:
                result.extend(self[0]._get_descendants_starting_with())
        return result

    def _get_descendants_stopping_with(self):
        from abjad.tools import scoretools
        result = []
        result.append(self)
        if isinstance(self, scoretools.Container):
            if self.is_simultaneous:
                for x in self:
                    result.extend(x._get_descendants_stopping_with())
            elif self:
                result.extend(self[-1]._get_descendants_stopping_with())
        return result

    def _get_duration(self, in_seconds=False):
        if in_seconds:
            return self._duration_in_seconds
        else:
            parentage = self._get_parentage(include_self=False)
            return parentage.prolation * self._preprolated_duration

    def _get_effective(self, prototype=None, unwrap=True, n=0):
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        # return time signature attached to measure regardless of scope
        if (
            prototype == indicatortools.TimeSignature or
            prototype == (indicatortools.TimeSignature,)
            ):
            if isinstance(self, scoretools.Measure):
                if self._has_indicator(indicatortools.TimeSignature):
                    indicator = self._get_indicator(
                        indicatortools.TimeSignature)
                    return indicator
                else:
                    return
        # update indicators of entire score tree if necessary
        self._update_now(indicators=True)
        # gather candidate expressions
        candidate_expressions = {}
        for parent in self._get_parentage(
            include_self=True,
            with_grace_notes=True,
            ):
            for indicator_expression in parent._dependent_expressions:
                if isinstance(indicator_expression.indicator, prototype):
                    offset = indicator_expression.start_offset
                    candidate_expressions.setdefault(offset, []).append(
                        indicator_expression)
            for indicator_expression in parent._indicator_expressions:
                if indicator_expression.scope is not None:
                    continue
                if isinstance(indicator_expression.indicator, prototype):
                    offset = indicator_expression.start_offset
                    candidate_expressions.setdefault(offset, []).append(
                        indicator_expression)
        if not candidate_expressions:
            return
        # elect most recent candidate expression
        all_offsets = sorted(candidate_expressions)
        start_offset = self._get_timespan()._start_offset
        index = bisect.bisect(all_offsets, start_offset) - 1 + int(n)
        if index < 0:
            return
        elif len(candidate_expressions) <= index:
            return
        expression = candidate_expressions[all_offsets[index]][0]
        if unwrap:
            expression = expression.indicator
        return expression

    def _get_effective_staff(self):
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        staff_change = self._get_effective(
            indicatortools.StaffChange)
        if staff_change is not None:
            effective_staff = staff_change.staff
        else:
            parentage = self._get_parentage()
            effective_staff = parentage.get_first(scoretools.Staff)
        return effective_staff

    def _get_format_contributions_for_slot(
        self,
        slot_identifier,
        bundle=None
        ):
        result = []
        if bundle is None:
            manager = systemtools.LilyPondFormatManager
            bundle = manager.bundle_format_contributions(self)
        slot_names = (
            'before',
            'open_brackets',
            'opening',
            'contents',
            'closing',
            'close_brackets',
            'after',
            )
        if isinstance(slot_identifier, int):
            assert slot_identifier in range(1, 7 + 1)
            slot_index = slot_identifier - 1
            slot_name = slot_names[slot_index]
        elif isinstance(slot_identifier, str):
            slot_name = slot_identifier.replace(' ', '_')
            assert slot_name in slot_names
        method_name = '_format_{}_slot'.format(slot_name)
        method = getattr(self, method_name)
        for source, contributions in method(bundle):
            result.extend(contributions)
        return result

    def _get_format_pieces(self):
        return self._format_component(pieces=True)

    def _get_format_specification(self):
        values = []
        summary = self._contents_summary
        if summary:
            values.append(summary)
        return systemtools.FormatSpecification(
            client=self,
            repr_args_values=values,
            storage_format_kwargs_names=[]
            )

    def _get_grace_containers(self, kind=None):
        from abjad.tools import scoretools
        result = []
        if (
            kind in (None, 'grace') and
            getattr(self, '_grace', None) is not None
            ):
            result.append(self._grace)
        if (
            kind in (None, 'after') and
            getattr(self, '_after_grace', None) is not None
            ):
            result.append(self._after_grace)
        elif kind == scoretools.GraceContainer:
            if self._grace is not None:
                result.append(self._grace)
            if self._after_grace is not None:
                result.append(self._after_grace)
        elif isinstance(kind, scoretools.GraceContainer):
            if self._grace is not None:
                result.append(self._grace)
            if self._after_grace is not None:
                result.append(self._after_grace)
        return tuple(result)

    def _get_in_my_logical_voice(self, n, prototype=None):
        if 0 <= n:
            generator = iterate(self).by_logical_voice_from_component(
                prototype=prototype,
                reverse=False,
                )
            for i, component in enumerate(generator):
                if i == n:
                    return component
        else:
            n = abs(n)
            generator = iterate(self).by_logical_voice_from_component(
                prototype=prototype,
                reverse=True,
                )
            for i, component in enumerate(generator):
                if i == n:
                    return component

    def _get_indicator(self, prototype=None, unwrap=True):
        indicators = self._get_indicators(prototype=prototype, unwrap=unwrap)
        if not indicators:
            message = 'no attached indicators found matching {!r}.'
            message = message.format(prototype)
            raise ValueError(message)
        elif 1 < len(indicators):
            message = 'multiple attached indicators found matching {!r}.'
            message = message.format(prototype)
            raise ValueError(message)
        else:
            return indicators[0]

    def _get_indicators(self, prototype=None, unwrap=True):
        from abjad.tools import indicatortools
        prototype = prototype or (object,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        prototype_objects, prototype_classes = [], []
        for indicator_prototype in prototype:
            if isinstance(indicator_prototype, type):
                prototype_classes.append(indicator_prototype)
            else:
                prototype_objects.append(indicator_prototype)
        prototype_objects = tuple(prototype_objects)
        prototype_classes = tuple(prototype_classes)
        matching_indicators = []
        for indicator in self._indicator_expressions:
            if isinstance(indicator, prototype_classes):
                matching_indicators.append(indicator)
            elif any(indicator == x for x in prototype_objects):
                matching_indicators.append(indicator)
            elif isinstance(indicator, indicatortools.IndicatorExpression):
                if isinstance(indicator.indicator, prototype_classes):
                    matching_indicators.append(indicator)
                elif any(indicator.indicator == x for x in prototype_objects):
                    matching_indicators.append(indicator)
        if unwrap:
            matching_indicators = [x.indicator for x in matching_indicators]
        matching_indicators = tuple(matching_indicators)
        return matching_indicators

    def _get_lilypond_format(self):
        self._update_now(indicators=True)
        return self._format_component()

    def _get_lineage(self):
        return selectiontools.Lineage(self)

    def _get_markup(self, direction=None):
        from abjad.tools import markuptools
        markup = self._get_indicators(markuptools.Markup)
        if direction == Up:
            return tuple(x for x in markup if x.direction == Up)
        elif direction == Down:
            return tuple(x for x in markup if x.direction == Down)
        return markup

    def _get_nth_component_in_time_order_from(self, n):
        assert mathtools.is_integer_equivalent(n)

        def next(component):
            if component is not None:
                for parent in component._get_parentage(include_self=True):
                    next_sibling = parent._get_sibling(1)
                    if next_sibling is not None:
                        return next_sibling

        def previous(component):
            if component is not None:
                for parent in component._get_parentage(include_self=True):
                    next_sibling = parent._get_sibling(-1)
                    if next_sibling is not None:
                        return next_sibling
        result = self
        if 0 < n:
            for i in range(n):
                result = next(result)
        elif n < 0:
            for i in range(abs(n)):
                result = previous(result)
        return result

    def _get_parentage(self, include_self=True, with_grace_notes=False):
        return selectiontools.Parentage(
            self,
            include_self=include_self,
            with_grace_notes=with_grace_notes,
            )

    def _get_sibling(self, n):
        if n == 0:
            return self
        elif 0 < n:
            if self._parent is not None:
                if not self._parent.is_simultaneous:
                    index = self._parent.index(self)
                    if index + n < len(self._parent):
                        return self._parent[index + n]
        elif n < 0:
            if self._parent is not None:
                if not self._parent.is_simultaneous:
                    index = self._parent.index(self)
                    if 0 <= index + n:
                        return self._parent[index + n]

    def _get_spanner(self, prototype=None, in_parentage=False):
        spanners = self._get_spanners(
            prototype=prototype,
            in_parentage=in_parentage,
            )
        if not spanners:
            message = 'no spanner found.'
            raise MissingSpannerError(message)
        elif len(spanners) == 1:
            return spanners.pop()
        else:
            message = 'multiple spanners found: {!r}'.format(spanners)
            raise ExtraSpannerError(message)

    def _get_spanner_indicators(self, prototype=None, unwrap=True):
        matching_indicators = []
        for spanner in self._get_spanners():
            result = spanner._get_indicators(
                prototype=prototype,
                unwrap=unwrap,
                )
            matching_indicators.extend(result)
        return matching_indicators

    def _get_spanners(
        self,
        prototype=None,
        in_parentage=False,
        ):
        from abjad.tools import spannertools
        prototype = prototype or (spannertools.Spanner,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        spanner_items = prototype[:]
        prototype, spanner_objects = [], []
        for spanner_item in spanner_items:
            if isinstance(spanner_item, type):
                prototype.append(spanner_item)
            elif isinstance(spanner_item, spannertools.Spanner):
                spanner_objects.append(spanner_item)
            else:
                message = 'must be spanner class or spanner object: {!r}'
                message = message.format(spanner_item)
        prototype = tuple(prototype)
        spanner_objects = tuple(spanner_objects)
        matching_spanners = set()
        in_parentage = bool(in_parentage)
        if in_parentage:
            components = self._get_parentage(include_self=True)
        else:
            components = (self,)
        for component in components:
            for spanner in set(component._spanners):
                if isinstance(spanner, prototype):
                    matching_spanners.add(spanner)
                elif any(spanner == x for x in spanner_objects):
                    matching_spanners.add(spanner)
        return matching_spanners

    def _get_timespan(self, in_seconds=False):
        if in_seconds:
            self._update_now(offsets_in_seconds=True)
            if self._start_offset_in_seconds is None:
                raise MissingTempoError
            return timespantools.Timespan(
                start_offset=self._start_offset_in_seconds,
                stop_offset=self._stop_offset_in_seconds,
                )
        else:
            self._update_now(offsets=True)
            return self._timespan

    def _get_vertical_moment(self, governor=None):
        offset = self._get_timespan()._start_offset
        if governor is None:
            governor = self._get_parentage().root
        return selectiontools.VerticalMoment(governor, offset)

    def _get_vertical_moment_at(self, offset):
        return selectiontools.VerticalMoment(self, offset)

    def _has_effective_indicator(self, prototype=None):
        indicator = self._get_effective(prototype=prototype)
        return indicator is not None

    def _has_indicator(self, prototype=None):
        indicators = self._get_indicators(prototype=prototype)
        return bool(indicators)

    def _has_spanner(
        self,
        prototype=None,
        in_parentage=False,
        ):
        spanners = self._get_spanners(
            prototype=prototype,
            in_parentage=in_parentage,
            )
        return bool(spanners)

    def _is_immediate_temporal_successor_of(self, component):
        temporal_successors = []
        current = self
        while current is not None:
            next_sibling = current._get_sibling(1)
            if next_sibling is None:
                current = current._parent
            else:
                temporal_successors = \
                    next_sibling._get_descendants_starting_with()
                break
        return component in temporal_successors

    def _move_indicators(self, recipient_component):
        for indicator in self._get_indicators(unwrap=False):
            detach(indicator, self)
            attach(indicator, recipient_component)

    # TODO: eventually reimplement as a keyword option to remove()
    def _remove_and_shrink_durated_parent_containers(self):
        from abjad.tools import indicatortools
        from abjad.tools import scoretools
        prolated_leaf_duration = self._get_duration()
        parentage = self._get_parentage(include_self=False)
        prolations = parentage._prolations
        current_prolation, i = durationtools.Duration(1), 0
        parent = self._parent
        while parent is not None and not parent.is_simultaneous:
            current_prolation *= prolations[i]
            if isinstance(parent, scoretools.FixedDurationTuplet):
                candidate_new_parent_dur = (parent.target_duration -
                    current_prolation * self.written_duration)
                if durationtools.Duration(0) < candidate_new_parent_dur:
                    parent.target_duration = candidate_new_parent_dur
            elif isinstance(parent, scoretools.Measure):
                indicator = parent._get_indicator(indicatortools.TimeSignature)
                parent_time_signature = indicator
                old_prolation = parent_time_signature.implied_prolation
                naive_time_signature = (
                    parent_time_signature.duration - prolated_leaf_duration)
                better_time_signature = mathtools.NonreducedFraction(
                    naive_time_signature)
                better_time_signature = better_time_signature.with_denominator(
                    parent_time_signature.denominator)
                better_time_signature = indicatortools.TimeSignature(
                    better_time_signature)
                detach(indicatortools.TimeSignature, parent)
                attach(better_time_signature, parent)
                indicator = parent._get_indicator(indicatortools.TimeSignature)
                parent_time_signature = indicator
                new_prolation = parent_time_signature.implied_prolation
                adjusted_prolation = old_prolation / new_prolation
                for x in parent:
                    if isinstance(x, scoretools.FixedDurationTuplet):
                        x.target_duration *= adjusted_prolation
                    else:
                        if adjusted_prolation != 1:
                            new_target = \
                                x._preprolated_duration * adjusted_prolation
                            scoretools.FixedDurationTuplet(new_target, [x])
            parent = parent._parent
            i += 1
        parentage = self._get_parentage(include_self=False)
        parent = self._parent
        if parent:
            index = parent.index(self)
            del(parent[index])
        for x in parentage:
            if not len(x):
                x._extract()
            else:
                break

    def _remove_from_parent(self):
        self._update_later(offsets=True)
        if self._parent is not None:
            self._parent._music.remove(self)
        self._parent = None

    def _remove_named_children_from_parentage(self, name_dictionary):
        if self._parent is not None and name_dictionary:
            for parent in self._get_parentage(include_self=False):
                named_children = parent._named_children
                for name in name_dictionary:
                    for component in name_dictionary[name]:
                        named_children[name].remove(component)
                    if not named_children[name]:
                        del named_children[name]

    def _restore_named_children_to_parentage(self, name_dictionary):
        if self._parent is not None and name_dictionary:
            for parent in self._get_parentage(include_self=False):
                named_children = parent._named_children
                for name in name_dictionary:
                    if name in named_children:
                        named_children[name].extend(name_dictionary[name])
                    else:
                        named_children[name] = copy.copy(name_dictionary[name])

    def _set_parent(self, new_parent):
        r'''Not composer-safe.
        '''
        named_children = self._cache_named_children()
        self._remove_named_children_from_parentage(named_children)
        self._remove_from_parent()
        self._parent = new_parent
        self._restore_named_children_to_parentage(named_children)
        self._update_later(offsets=True)

    def _splice(
        self,
        components,
        direction=Right,
        grow_spanners=True,
        ):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        assert all(isinstance(x, scoretools.Component) for x in components)
        selection = selectiontools.Selection(self)
        if direction == Right:
            if grow_spanners:
                insert_offset = self._get_timespan()._stop_offset
                receipt = selection._get_dominant_spanners()
                for spanner, index in receipt:
                    insert_component = None
                    for component in spanner:
                        start_offset = component._get_timespan()._start_offset
                        if start_offset == insert_offset:
                            insert_component = component
                            break
                    if insert_component is not None:
                        insert_index = spanner._index(insert_component)
                    else:
                        insert_index = len(spanner)
                    for component in reversed(components):
                        spanner._insert(insert_index, component)
                        component._spanners.add(spanner)
            selection = selectiontools.Selection(self)
            parent, start, stop = \
                selection._get_parent_and_start_stop_indices()
            if parent is not None:
                if grow_spanners:
                    for component in reversed(components):
                        component._set_parent(parent)
                        parent._music.insert(start + 1, component)
                else:
                    after = stop + 1
                    parent.__setitem__(slice(after, after), components)
            return [self] + components
        else:
            if grow_spanners:
                offset = self._get_timespan()._start_offset
                receipt = selection._get_dominant_spanners()
                for spanner, x in receipt:
                    for component in spanner:
                        if component._get_timespan()._start_offset == offset:
                            index = spanner._index(component)
                            break
                    else:
                        message = 'no component in spanner at offset.'
                        raise ValueError(message)
                    for component in reversed(components):
                        spanner._insert(index, component)
                        component._spanners.add(spanner)
            selection = selectiontools.Selection(self)
            parent, start, stop = \
                selection._get_parent_and_start_stop_indices()
            if parent is not None:
                if grow_spanners:
                    for component in reversed(components):
                        component._set_parent(parent)
                        parent._music.insert(start, component)
                else:
                    parent.__setitem__(slice(start, start), components)
            return components + [self]

    def _update_later(self, offsets=False, offsets_in_seconds=False):
        assert offsets or offsets_in_seconds
        for component in self._get_parentage(include_self=True):
            if offsets:
                component._offsets_are_current = False
            elif offsets_in_seconds:
                component._offsets_in_seconds_are_current = False

    def _update_logical_measure_numbers(self):
        from abjad.tools import systemtools
        update_manager = systemtools.UpdateManager()
        update_manager._update_logical_measure_numbers(self)

    def _update_now(
        self,
        offsets=False,
        offsets_in_seconds=False,
        indicators=False,
        ):
        from abjad.tools import systemtools
        update_manager = systemtools.UpdateManager()
        return update_manager._update_now(
            self,
            offsets=offsets,
            offsets_in_seconds=offsets_in_seconds,
            indicators=indicators,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def name(self):
        r'''Gets and sets name of component.

        Returns string or none.
        '''
        return self._name

    @name.setter
    def name(self, arg):
        assert isinstance(arg, (str, type(None)))
        old_name = self._name
        for parent in self._get_parentage(include_self=False):
            named_children = parent._named_children
            if old_name is not None:
                named_children[old_name].remove(self)
                if not named_children[old_name]:
                    del named_children[old_name]
            if arg is not None:
                if arg not in named_children:
                    named_children[arg] = [self]
                else:
                    named_children[arg].append(self)
        self._name = arg



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class QTargetMeasure(AbjadObject):
    r'''Representation of a single "measure" in a measure-wise
    quantization target:

    ::

        >>> search_tree = quantizationtools.UnweightedSearchTree({2: None})
        >>> tempo = Tempo((1, 4), 60)
        >>> time_signature = TimeSignature((4, 4))

    ::

        >>> q_target_measure = quantizationtools.QTargetMeasure(
        ...     offset_in_ms=1000,
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     time_signature=time_signature,
        ...     )

    ::

        >>> print(format(q_target_measure, 'storage'))
        quantizationtools.QTargetMeasure(
            offset_in_ms=durationtools.Offset(1000, 1),
            search_tree=quantizationtools.UnweightedSearchTree(
                definition={   2: None,
                    },
                ),
            time_signature=indicatortools.TimeSignature((4, 4)),
            tempo=indicatortools.Tempo(
                reference_duration=durationtools.Duration(1, 4), 
                units_per_minute=60,
                ),
            use_full_measure=False,
            )

    ``QTargetMeasures`` group ``QTargetBeats``:

    ::

        >>> for q_target_beat in q_target_measure.beats:
        ...     print(q_target_beat.offset_in_ms, q_target_beat.duration_in_ms)
        1000 1000
        2000 1000
        3000 1000
        4000 1000

    If ``use_full_measure`` is set, the ``QTargetMeasure`` will only
    ever contain a single ``QTargetBeat`` instance:

    ::

        >>> another_q_target_measure = quantizationtools.QTargetMeasure(
        ...     offset_in_ms=1000,
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     time_signature=time_signature,
        ...     use_full_measure=True,
        ...     )

    ::

        >>> for q_target_beat in another_q_target_measure.beats:
        ...     print(q_target_beat.offset_in_ms, q_target_beat.duration_in_ms)
        1000 4000

    Not composer-safe.

    Used internally by ``Quantizer``.

    Return ``QTargetMeasure`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beats',
        '_offset_in_ms',
        '_search_tree',
        '_tempo',
        '_time_signature',
        '_use_full_measure',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        offset_in_ms=None,
        search_tree=None,
        time_signature=None,
        tempo=None,
        use_full_measure=False,
        ):

        from abjad.tools import quantizationtools

        offset_in_ms = offset_in_ms or 0
        offset_in_ms = durationtools.Offset(offset_in_ms)

        if search_tree is None:
            search_tree = quantizationtools.UnweightedSearchTree()
        assert isinstance(search_tree, quantizationtools.SearchTree)
        tempo = tempo or indicatortools.Tempo((1, 4), 60)
        #tempo = indicatortools.Tempo(tempo)
        if isinstance(tempo, tuple):
            tempo = indicatortools.Tempo(*tempo)
        assert not tempo.is_imprecise
        time_signature = time_signature or (4, 4)
        time_signature = indicatortools.TimeSignature(time_signature)
        use_full_measure = bool(use_full_measure)

        beats = []

        if use_full_measure:
            beatspan = time_signature.duration
            beat = quantizationtools.QTargetBeat(
                beatspan=beatspan,
                offset_in_ms=offset_in_ms,
                search_tree=search_tree,
                tempo=tempo
                )
            beats.append(beat)
        else:
            beatspan = durationtools.Duration(1, time_signature.denominator)
            current_offset_in_ms = offset_in_ms
            beatspan_duration_in_ms = \
                tempo.duration_to_milliseconds(beatspan)
            for i in range(time_signature.numerator):
                beat = quantizationtools.QTargetBeat(
                    beatspan=beatspan,
                    offset_in_ms=current_offset_in_ms,
                    search_tree=search_tree,
                    tempo=tempo
                    )
                beats.append(beat)
                current_offset_in_ms += beatspan_duration_in_ms

        self._beats = tuple(beats)
        self._offset_in_ms = offset_in_ms
        self._search_tree = search_tree
        self._tempo = tempo
        self._time_signature = time_signature
        self._use_full_measure = use_full_measure

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats q-event.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    ### PUBLIC PROPERTIES ###

    @property
    def beats(self):
        r'''The tuple of ``QTargetBeats`` contained by the ``QTargetMeasure``:

        ::

            >>> for q_target_beat in q_target_measure.beats:
            ...     print(format(q_target_beat, 'storage'))
            ... 
            quantizationtools.QTargetBeat(
                beatspan=durationtools.Duration(1, 4),
                offset_in_ms=durationtools.Offset(1000, 1),
                search_tree=quantizationtools.UnweightedSearchTree(
                    definition={   2: None,
                        },
                    ),
                tempo=indicatortools.Tempo(
                    reference_duration=durationtools.Duration(1, 4), 
                    units_per_minute=60,
                    ),
                )
            quantizationtools.QTargetBeat(
                beatspan=durationtools.Duration(1, 4),
                offset_in_ms=durationtools.Offset(2000, 1),
                search_tree=quantizationtools.UnweightedSearchTree(
                    definition={   2: None,
                        },
                    ),
                tempo=indicatortools.Tempo(
                    reference_duration=durationtools.Duration(1, 4), 
                    units_per_minute=60,
                    ),
                )
            quantizationtools.QTargetBeat(
                beatspan=durationtools.Duration(1, 4),
                offset_in_ms=durationtools.Offset(3000, 1),
                search_tree=quantizationtools.UnweightedSearchTree(
                    definition={   2: None,
                        },
                    ),
                tempo=indicatortools.Tempo(
                    reference_duration=durationtools.Duration(1, 4), 
                    units_per_minute=60,
                    ),
                )
            quantizationtools.QTargetBeat(
                beatspan=durationtools.Duration(1, 4),
                offset_in_ms=durationtools.Offset(4000, 1),
                search_tree=quantizationtools.UnweightedSearchTree(
                    definition={   2: None,
                        },
                    ),
                tempo=indicatortools.Tempo(
                    reference_duration=durationtools.Duration(1, 4), 
                    units_per_minute=60,
                    ),
                )

        Returns tuple.
        '''

        return self._beats

    @property
    def duration_in_ms(self):
        r'''The duration in milliseconds of the ``QTargetMeasure``:

        ::

            >>> q_target_measure.duration_in_ms
            Duration(4000, 1)

        Returns Duration.
        '''
        from abjad.tools import quantizationtools
        return self.tempo.duration_to_milliseconds(
            self.time_signature.duration)

    @property
    def offset_in_ms(self):
        r'''The offset in milliseconds of the ``QTargetMeasure``:

        ::

            >>> q_target_measure.offset_in_ms
            Offset(1000, 1)

        Returns Offset.
        '''
        return self._offset_in_ms

    @property
    def search_tree(self):
        r'''The search tree of the ``QTargetMeasure``:

        ::

            >>> q_target_measure.search_tree
            UnweightedSearchTree(definition={2: None})

        Return ``SearchTree`` instance.
        '''
        return self._search_tree

    @property
    def tempo(self):
        r'''The tempo of the ``QTargetMeasure``:

        ::

            >>> q_target_measure.tempo
            Tempo(reference_duration=Duration(1, 4), units_per_minute=60)

        Return ``Tempo`` instance.
        '''
        return self._tempo

    @property
    def time_signature(self):
        r'''The time signature of the ``QTargetMeasure``:

        ::

            >>> q_target_measure.time_signature
            TimeSignature((4, 4))

        Return ```TimeSignature`` instance.
        '''
        return self._time_signature

    @property
    def use_full_measure(self):
        r'''The ``use_full_measure`` flag of the ``QTargetMeasure``:

        ::

            >>> q_target_measure.use_full_measure
            False

        Returns true or false.
        '''
        return self._use_full_measure



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools


def shadow_pitch_contour_reservoir(pitch_contour_reservoir):
    r'''Shadows pitch contour reservoir.
    '''

    shadow_pitch_lookup = {
        pitchtools.NamedPitchClass('a'): -5, # add a P4 below
        pitchtools.NamedPitchClass('g'): -3, # add a m3 below
        pitchtools.NamedPitchClass('f'): -1, # add a m2 below
        pitchtools.NamedPitchClass('e'): -4, # add a M3 below
        pitchtools.NamedPitchClass('d'): -2, # add a M2 below
        pitchtools.NamedPitchClass('c'): -3, # add a m3 below
        pitchtools.NamedPitchClass('b'): -2, # add a M2 below
    }

    shadowed_reservoir = {}

    for instrument_name, pitch_contours in pitch_contour_reservoir.items():
        # The viola does not receive any diads
        if instrument_name == 'Viola':
            shadowed_reservoir['Viola'] = pitch_contours
            continue

        shadowed_pitch_contours = []

        for pitch_contour in pitch_contours[:-1]:
            shadowed_pitch_contour = []
            for pitch in pitch_contour:
                pitch_class = pitch.named_pitch_class
                shadow_pitch = pitch + shadow_pitch_lookup[pitch_class]
                diad = (shadow_pitch, pitch)
                shadowed_pitch_contour.append(diad)
            shadowed_pitch_contours.append(tuple(shadowed_pitch_contour))

        # treat the final contour differently: the last note does not become a diad
        final_shadowed_pitch_contour = []
        for pitch in pitch_contours[-1][:-1]:
            pitch_class = pitch.named_pitch_class
            shadow_pitch = pitch + shadow_pitch_lookup[pitch_class]
            diad = (shadow_pitch, pitch)
            final_shadowed_pitch_contour.append(diad)
        final_shadowed_pitch_contour.append(pitch_contours[-1][-1])
        shadowed_pitch_contours.append(tuple(final_shadowed_pitch_contour))

        shadowed_reservoir[instrument_name] = tuple(shadowed_pitch_contours)

    return shadowed_reservoir



# -*- coding: utf-8 -*-
import difflib
import inspect
import os
from abjad.tools.abctools import AbjadObject


class TestManager(AbjadObject):
    r'''Manages test logic.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Managers'

    ### PRIVATE METHODS ###

    @staticmethod
    def _compare_backup(path):
        if isinstance(path, str):
            paths = [path]
        elif (isinstance(path, (tuple, list))):
            paths = path
        else:
            raise TypeError(path)
        for path in paths:
            backup_path = path + '.backup'
            if not TestManager.compare_files(path, backup_path):
                return False
        return True

    @staticmethod
    def _compare_lys(path_1, path_2):
        r'''Compares LilyPond file `path_1` to LilyPond file `path_2`.

        Performs line-by-line comparison.

        Discards blank lines.

        Discards any LilyPond version statements.

        Discards any lines beginning with ``%``.

        Returns true or false.
        '''
        file_1_lines = TestManager._normalize_ly(path_1)
        file_2_lines = TestManager._normalize_ly(path_2)
        return file_1_lines == file_2_lines

    @staticmethod
    def _compare_pdfs(path_1, path_2):
        r'''Compares PDF `path_1` to PDF `path_2`.

        Performs line-by-line comparison.

        Discards blank lines.

        Discards lines that contain any of the following strings:

        * ``/ID``
        * ``/CreationDate``
        * ``/ModDate``
        * ``xmp:CreateDate``
        * ``xmp:ModifyDate``
        * ``xapMM:DocumentID``
        * ``rdf:about``

        Discards first (binary) stream object in PDF;
        possibly first stream contains binary-encoded creator
        or timestamp information that can vary from one creation
        of a PDF to another.

        Returns true or false.
        '''
        file_1_lines = TestManager._normalize_pdf(path_1)
        file_2_lines = TestManager._normalize_pdf(path_2)
        return file_1_lines == file_2_lines

    @staticmethod
    def _compare_text_files(path_1, path_2):
        r'''Compares text file `path_1` to text file `path_2`.

        Performs line-by-line comparison.

        Discards blank lines.

        Trims whitespace from the end of each line.

        Returns true or false.
        '''
        file_1_lines, file_2_lines = [], []
        with open(path_1, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                file_1_lines.append(line)
        with open(path_2, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                file_2_lines.append(line)
        return file_1_lines == file_2_lines

    @staticmethod
    def _get_first_differing_pdf_lines(path_1, path_2):
        file_1_lines = TestManager._normalize_pdf(path_1)
        file_2_lines = TestManager._normalize_pdf(path_2)
        for file_1_line, file_2_line in zip(file_1_lines, file_2_lines):
            if not file_1_line == file_2_line:
                return file_1_line, file_2_line

    @staticmethod
    def _normalize_ly(path):
        lines = []
        with open(path, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                if line.startswith(r'\version'):
                    continue
                elif line.startswith('%'):
                    continue
                lines.append(line)
        return lines

    @staticmethod
    def _normalize_pdf(path):
        lines = []
        with open(path, 'rb') as file_pointer:
            contents = file_pointer.read()
            for line in contents.splitlines():
                line = line.strip()
                if line == b'':
                    continue
                elif line.startswith(b'%'):
                    continue
                elif b'/ID' in line:
                    continue
                elif b'/Creator' in line:
                    continue
                elif b'/CreationDate' in line:
                    continue
                elif b'/ModDate' in line:
                    continue
                elif b'xmp:CreateDate' in line:
                    continue
                elif b'xmp:CreatorTool' in line:
                    continue
                elif b'xmp:ModifyDate' in line:
                    continue
                elif b'xapMM:DocumentID' in line:
                    continue
                elif b'rdf:about' in line:
                    continue
                lines.append(line)
        # discard first stream in document: can contain differences;
        # no idea why; possibly first stream includes binary-encoded
        # creator information or binary-encoded timestamp information
        new_lines = []
        found_first_stream = False
        in_first_stream = False
        for line in lines:
            if line == b'stream' and not found_first_stream:
                found_first_stream = True
                in_first_stream = True
            elif line.endswith(b'endstream') and in_first_stream:
                in_first_stream = False
                continue
            if in_first_stream:
                continue
            else:
                new_lines.append(line)
        return new_lines

    ### PUBLIC METHODS ###

    @staticmethod
    def apply_additional_layout(lilypond_file):
        r'''Configures multiple-voice rhythmic staves in `lilypond_file`.

        Operates in place.

        Returns none.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import scoretools
        from abjad.tools import topleveltools
        # configure multiple-voice rhythmic staves
        score = lilypond_file[Score]
        for staff in topleveltools.iterate(score).by_class(scoretools.Staff):
            if staff.is_simultaneous:
                assert len(staff) == 2
                voice_1 = staff[0]
                topleveltools.override(voice_1).note_head.Y_offset = 0.5
                topleveltools.override(voice_1).stem.direction = Up
                voice_2 = staff[1]
                topleveltools.override(voice_2).note_head.Y_offset = -0.5
                topleveltools.override(voice_2).stem.direction = Down
                spacing_vector = schemetools.make_spacing_vector(0, 0, 6, 0)
                manager = topleveltools.override(staff)
                manager.vertical_axis_group.staff_staff_spacing = spacing_vector
        # provide more space between staves with pitched notes
        score = lilypond_file[Score]
        for staff in topleveltools.iterate(score).by_class(scoretools.Staff):
            if not (isinstance(staff, scoretools.Staff) and
                staff.context_name == 'RhythmicStaff'):
                for item in lilypond_file.layout_block.items:
                    if isinstance(item, lilypondfiletools.ContextBlock):
                        if item.source_context_name == 'StaffGroup':
                            break
                else:
                    message = 'no staff group context block found.'
                    raise Exception(message)
                spacing_vector = schemetools.make_spacing_vector(0, 0, 6, 0)
                manager = topleveltools.override(item)
                manager.vertical_axis_group.staff_staff_spacing = spacing_vector
            break

    @staticmethod
    def clean_string(string):
        r'''Cleans string.
        '''
        from abjad.tools import stringtools
        return stringtools.normalize(string)

    @staticmethod
    def compare(string_1, string_2):
        r'''Compares `string_1` to `string_2`.

        Massage newlines.

        Returns true or false.
        '''
        if not isinstance(string_1, str):
            string_1 = format(string_1)
        split_lines = string_2.split('\n')
        if not split_lines[0] or split_lines[0].isspace():
            split_lines.pop(0)
        if not split_lines[-1] or split_lines[-1].isspace():
            split_lines.pop(-1)
        indent_width = 0
        if split_lines:
            for indent_width, character in enumerate(split_lines[0]):
                if character != ' ':
                    break
        massaged_lines = []
        for split_line in split_lines:
            massaged_line = split_line[indent_width:]
            massaged_lines.append(massaged_line)
        massaged_string = '\n'.join(massaged_lines)
        return string_1.replace('\t', '    ') == massaged_string

    @staticmethod
    def compare_files(path_1, path_2):
        r'''Compares file `path_1` to file `path_2`.

        For all file types::

        * Performs line-by-line comparison
        * Discards blank lines

        For LilyPond files, additionally::

        * Discards any LilyPond version statements
        * Discards any lines beginning with ``%``

        For PDFs, additionally discards lines that contain any of the
        following strings:

        * ``/ID``
        * ``/CreationDate``
        * ``/ModDate``
        * ``xmp:CreateDate``
        * ``xmp:ModifyDate``
        * ``xapMM:DocumentID``
        * ``rdf:about``

        Discards first (binary) stream object in PDF;
        possibly first stream contains binary-encoded creator
        or timestamp information that can vary from one creation
        of a PDF to another.

        Returns true when files compare the same and false when files compare
        differently.
        '''
        if os.path.exists(path_1) and not os.path.exists(path_2):
            return False
        elif not os.path.exists(path_1) and os.path.exists(path_2):
            return False
        elif not os.path.exists(path_1) and not os.path.exists(path_2):
            return True
        if path_1.endswith('.backup'):
            path_1 = path_1.strip('.backup')
        if path_2.endswith('.backup'):
            path_2 = path_2.strip('.backup')
        base_1, extension_1 = os.path.splitext(path_1)
        base_2, extension_2 = os.path.splitext(path_2)
        assert extension_1 == extension_2
        if extension_1 == '.ly':
            return TestManager._compare_lys(path_1, path_2)
        elif extension_1 == '.pdf':
            return TestManager._compare_pdfs(path_1, path_2)
        else:
            return TestManager._compare_text_files(path_1, path_2)

    @staticmethod
    def compare_objects(object_one, object_two):
        r'''Compares `object_one` to `object_two`.

        Returns true or false.
        '''
        from abjad.tools import systemtools
        agent_one = systemtools.StorageFormatAgent(object_one)
        if agent_one.format_specification.coerce_for_equality:
            try:
                object_two = type(object_one)(object_two)
            except (
                TypeError,
                ValueError,
                ):
                return False
        elif not isinstance(object_two, type(object_one)):
            return False
        agent_two = systemtools.StorageFormatAgent(object_two)
        return agent_one.get_template_dict() == agent_two.get_template_dict()

    @staticmethod
    def diff(object_a, object_b, title=None):
        r'''Gets diff of `object_a` and `object_b` formats.

        ::

            >>> one = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=rhythmmakertools.Talea(
            ...         counts=[1, 2, 3],
            ...         denominator=8,
            ...         )
            ...     )

        ::

            >>> two = rhythmmakertools.TaleaRhythmMaker(
            ...     talea=rhythmmakertools.Talea(
            ...         counts=[1, 5, 3],
            ...         denominator=4,
            ...         )
            ...     )

        ::

            >>> diff = systemtools.TestManager.diff(one, two, 'Diff:')
            >>> print(diff)
            Diff:
              rhythmmakertools.TaleaRhythmMaker(
                  talea=rhythmmakertools.Talea(
            -         counts=[1, 2, 3],
            ?                    ^
            +         counts=[1, 5, 3],
            ?                    ^
            -         denominator=8,
            ?                     ^
            +         denominator=4,
            ?                     ^
                      ),
                  )

        Returns string.
        '''
        try:
            a_format = format(object_a, 'storage')
        except ValueError:
            a_format = format(object_a)
        try:
            b_format = format(object_b, 'storage')
        except ValueError:
            b_format = format(object_b)
        a_format = a_format.splitlines(True)
        b_format = b_format.splitlines(True)
        diff = ''.join(difflib.ndiff(a_format, b_format))
        if title is not None:
            diff = title + '\n' + diff
        return diff

    @staticmethod
    def get_current_function_name():
        r'''Gets current function name.

        ::

            >>> def foo():
            ...        function_name = systemtools.TestManager.get_current_function_name()
            ...        print('Function name is {!r}.'.format(function_name))

        ::

            >>> foo()
            Function name is 'foo'.

        Call this function within the implementation of any ofther function.

        Returns enclosing function name as a string or else none.
        '''
        stack = inspect.stack()
        # per the inspect module doc page ...
        try:
            parent_frame_record = stack[1]
            parent_frame = parent_frame_record[0]
            parent_frame_info = inspect.getframeinfo(parent_frame)
            parent_frame_function_name = parent_frame_info.function
            return parent_frame_function_name
        # ... destroy frame to avoid reference cycle
        finally:
            del stack

    @staticmethod
    def read_test_output(full_file_name, current_function_name):
        r'''Reads test output.

        Returns list.
        '''
        segment_ly_file_name = '{}.ly'.format(current_function_name)
        directory_name = os.path.dirname(full_file_name)
        segment_ly_path_name = os.path.join(
            directory_name,
            segment_ly_file_name,
            )
        with open(segment_ly_path_name, 'r') as f:
            string = f.read()
        return string

    @staticmethod
    def test_function_name_to_title_lines(test_function_name):
        r'''Changes `test_function_name` to title lines.

        Returns list.
        '''
        from abjad.tools import sequencetools
        title_lines = []
        test_function_name = test_function_name[5:]
        if '__' in test_function_name:
            left_half, right_half = test_function_name.split('__')
            left_half = left_half.replace('_', ' ')
            title_lines.append(left_half)
            parts = right_half.split('_')
        else:
            parts = test_function_name.split('_')
        test_number = int(parts[-1])
        parts.pop(-1)
        if parts[0][0].isupper() and 1 < len(parts[0]):
            title_lines.append(parts.pop(0))
        lengths = [len(part) for part in parts]
        if 35 < sum(lengths):
            halves = baca.Sequence(halves)
            halves = halves.partition_by_ratio_of_weights(ratio=[1, 1])
            left_count = len(halves[0])
            right_count = len(halves[-1])
            assert left_count + right_count == len(lengths)
            left_parts = parts[:left_count]
            title_lines.append(' '.join(left_parts))
            right_parts = parts[-right_count:]
            right_parts.append(str(test_number))
            title_lines.append(' '.join(right_parts))
        else:
            title_words = ' '.join(parts)
            if 'schematic example' in title_words:
                space = ''
            else:
                space = ' '
            title = '{}{}{}'.format(title_words, space, test_number)
            title_lines.append(title)
        return title_lines



# -*- coding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import select


def make_nested_tuplet(
    tuplet_duration,
    outer_tuplet_proportions,
    inner_tuplet_subdivision_count,
    ):
    r'''Makes nested tuplet.
    '''

    outer_tuplet = scoretools.Tuplet.from_duration_and_ratio(
        tuplet_duration, outer_tuplet_proportions)
    inner_tuplet_proportions = inner_tuplet_subdivision_count * [1]
    selector = select().by_leaf(flatten=True)
    last_leaf = selector(outer_tuplet)[-1]
    right_logical_tie = inspect_(last_leaf).get_logical_tie()
    right_logical_tie.to_tuplet(inner_tuplet_proportions)
    return outer_tuplet


# -*- coding: utf-8 -*-
import math
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import spannertools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import select


def make_desordre_cell(pitches):
    '''The function constructs and returns a *Désordre cell*.
    `pitches` is a list of numbers or, more generally, pitch tokens.
    '''

    notes = [scoretools.Note(pitch, (1, 8)) for pitch in pitches]
    beam = spannertools.Beam()
    attach(beam, notes)
    slur = spannertools.Slur()
    attach(slur, notes)
    clef = indicatortools.Dynamic('f')
    attach(clef, notes[0])
    dynamic = indicatortools.Dynamic('p')
    attach(dynamic, notes[1])

    # make the lower voice
    lower_voice = scoretools.Voice(notes)
    lower_voice.name = 'RH Lower Voice'
    command = indicatortools.LilyPondCommand('voiceTwo')
    attach(command, lower_voice)
    n = int(math.ceil(len(pitches) / 2.))
    chord = scoretools.Chord([pitches[0], pitches[0] + 12], (n, 8))
    articulation = indicatortools.Articulation('>')
    attach(articulation, chord)

    # make the upper voice
    upper_voice = scoretools.Voice([chord])
    upper_voice.name = 'RH Upper Voice'
    command = indicatortools.LilyPondCommand('voiceOne')
    attach(command, upper_voice)

    # combine them together
    container = scoretools.Container([lower_voice, upper_voice])
    container.is_simultaneous = True

    # make all 1/8 beats breakable
    leaves = select(lower_voice).by_leaf()
    for leaf in leaves[:-1]:
        bar_line = indicatortools.BarLine('')
        attach(bar_line, leaf)

    return container



# -*- coding: utf-8 -*-
from abjad import *


def test_schemetools_SchemeAssociativeList___init___01():

    scheme_associative_list = schemetools.SchemeAssociativeList(('space', 2), ('padding', 0.5))
    assert format(scheme_associative_list) == "#'((space . 2) (padding . 0.5))"



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_quantizationtools_QGridContainer___copy___01():

    tree = quantizationtools.QGridContainer(
        preprolated_duration=1,
        children=[
            quantizationtools.QGridLeaf(preprolated_duration=1),
            quantizationtools.QGridContainer(
                preprolated_duration=2,
                children=[
                    quantizationtools.QGridLeaf(preprolated_duration=3),
                    quantizationtools.QGridContainer(
                        preprolated_duration=4,
                        children=[
                            quantizationtools.QGridLeaf(preprolated_duration=1),
                            quantizationtools.QGridLeaf(preprolated_duration=1),
                            quantizationtools.QGridLeaf(preprolated_duration=1)
                        ])
                ]),
            quantizationtools.QGridLeaf(preprolated_duration=2)
        ])

    copied = copy.copy(tree)

    assert format(tree) == format(copied)
    assert tree is not copied
    assert tree[0] is not copied[0]
    assert tree[1] is not copied[1]
    assert tree[2] is not copied[2]



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QGridLeaf___eq___01():
    a = quantizationtools.QGridLeaf(1, [])
    b = quantizationtools.QGridLeaf(1, [])
    assert format(a) == format(b)
    assert a != b


def test_quantizationtools_QGridLeaf___eq___02():
    a = quantizationtools.QGridLeaf(1, [])
    b = quantizationtools.QGridLeaf(1, [quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1000), 0.5)])
    c = quantizationtools.QGridLeaf(2, [])
    d = quantizationtools.QGridLeaf(2, [quantizationtools.QEventProxy(quantizationtools.SilentQEvent(1000), 0.5)])
    assert a != b
    assert a != c
    assert a != d
    assert b != c
    assert b != d
    assert c != d



# -*- coding: utf-8 -*-
import abc


AbstractBase = abc.ABCMeta(
    'AbstractBase',
    (),
    {
        '__metaclass__': abc.ABCMeta,
        '__module__': __name__,
        '__slots__': (),
        },
    )


class AbjadObject(AbstractBase):
    '''Abstract base class from which many custom classes inherit.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when ID of `argument` equals ID of Abjad object.
        Otherwise false.

        Returns true or false.
        '''
        return id(self) == id(argument)

    def __format__(self, format_specification=''):
        r'''Formats Abjad object.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getstate__(self):
        r'''Gets state of Abjad object.

        Returns dictionary.
        '''
        if hasattr(self, '__dict__') and hasattr(vars(self), 'copy'):
            state = vars(self).copy()
        else:
            state = {}
        for class_ in type(self).__mro__:
            for slot in getattr(class_, '__slots__', ()):
                try:
                    state[slot] = getattr(self, slot)
                except AttributeError:
                    pass
        return state

    def __hash__(self):
        r'''Hashes Abjad object.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(AbjadObject, self).__hash__()

    def __ne__(self, argument):
        r'''Is true when Abjad object does not equal `argument`.
        Otherwise false.

        Returns true or false.
        '''
        return not self == argument

    def __repr__(self):
        r'''Gets interpreter representation of Abjad object.

        Returns string.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatAgent(self).get_repr_format()

    def __setstate__(self, state):
        r'''Sets state of Abjad object.

        Returns none.
        '''
        for key, value in state.items():
            setattr(self, key, value)

    ### PRIVATE METHODS ###

    def _debug(self, value, annotation=None, blank=False):
        if annotation is None:
            print('debug: {!r}'.format(value))
        else:
            print('debug ({}): {!r}'.format(annotation, value))
        if blank:
            print()

    def _debug_values(self, values, annotation=None, blank=True):
        if values:
            for value in values:
                self._debug(value, annotation=annotation)
            if blank:
                print()
        else:
            self._debug(repr(values), annotation=annotation)
            if blank:
                print()

    def _get_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.FormatSpecification(client=self)



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadValueObject


class InterpolationSpecifier(AbjadValueObject):
    r'''Interpolation specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_start_duration',
        '_stop_duration',
        '_written_duration',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        start_duration=durationtools.Duration(1, 8),
        stop_duration=durationtools.Duration(1, 16),
        written_duration=durationtools.Duration(1, 16),
        ):
        self._start_duration = durationtools.Duration(start_duration)
        self._stop_duration = durationtools.Duration(stop_duration)
        self._written_duration = durationtools.Duration(written_duration)

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Swaps start duration and stop duration of interpolation specifier.

        ..  container:: example

            Changes accelerando specifier to ritardando specifier:

            ::

                >>> specifier = rhythmmakertools.InterpolationSpecifier(
                ...     start_duration=Duration(1, 4),
                ...     stop_duration=Duration(1, 16),
                ...     written_duration=Duration(1, 16),
                ...     )
                >>> specifier = specifier.reverse()
                >>> f(specifier)
                rhythmmakertools.InterpolationSpecifier(
                    start_duration=durationtools.Duration(1, 16),
                    stop_duration=durationtools.Duration(1, 4),
                    written_duration=durationtools.Duration(1, 16),
                    )

        ..  container:: example

            Changes ritardando specifier to accelerando specifier:

            ::

                >>> specifier = rhythmmakertools.InterpolationSpecifier(
                ...     start_duration=Duration(1, 16),
                ...     stop_duration=Duration(1, 4),
                ...     written_duration=Duration(1, 16),
                ...     )
                >>> specifier = specifier.reverse()
                >>> f(specifier)
                rhythmmakertools.InterpolationSpecifier(
                    start_duration=durationtools.Duration(1, 4),
                    stop_duration=durationtools.Duration(1, 16),
                    written_duration=durationtools.Duration(1, 16),
                    )

        Copies written duration from source.
        '''
        return type(self)(
            start_duration=self.stop_duration,
            stop_duration=self.start_duration,
            written_duration=self.written_duration,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def start_duration(self):
        r'''Gets start duration.

        Defaults to ``1/8``.

        Set to positive duration.

        Returns position duration.
        '''
        return self._start_duration

    @property
    def stop_duration(self):
        r'''Gets stop duration.

        Defaults to ``1/16``.

        Set to positive duration.

        Returns position duration.
        '''
        return self._stop_duration

    @property
    def written_duration(self):
        r'''Gets written duration.

        Defaults to ``1/16``.

        Set to positive duration.

        Returns position duration.
        '''
        return self._written_duration



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import attach


def make_empty_piano_score():
    r'''Make empty piano score:

    ::

        >>> score, treble, bass = scoretools.make_empty_piano_score()

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                }
                \context Staff = "bass" {
                    \clef "bass"
                }
            >>
        >>

    Returns score, treble staff, bass staff.
    '''
    from abjad.tools import indicatortools
    from abjad.tools import scoretools

    # make treble staff
    treble_staff = scoretools.Staff([])
    treble_staff.name = 'treble'
    clef = indicatortools.Clef('treble')
    attach(clef, treble_staff)

    # make bass staff
    bass_staff = scoretools.Staff([])
    bass_staff.name = 'bass'
    clef = indicatortools.Clef('bass')
    attach(clef, bass_staff)

    # make piano staff and score
    staff_group = scoretools.StaffGroup([treble_staff, bass_staff])
    staff_group.context_name = 'PianoStaff'
    score = scoretools.Score([])
    score.append(staff_group)

    # return score, treble staff, bass staff
    return score, treble_staff, bass_staff



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class Quantizer(AbjadObject):
    r'''``Quantizer`` quantizes sequences of attack-points, encapsulated by
    ``QEventSequences``, into score trees.

    ::

        >>> quantizer = quantizationtools.Quantizer()

    ::

        >>> durations = [1000] * 8
        >>> pitches = range(8)
        >>> q_event_sequence = \
        ...     quantizationtools.QEventSequence.from_millisecond_pitch_pairs(
        ...     tuple(zip(durations, pitches)))

    Quantization defaults to outputting into a 4/4, quarter=60 musical
    structure:

    ::

        >>> result = quantizer(q_event_sequence)
        >>> score = Score([Staff([result])])
        >>> print(format(score))
        \new Score <<
            \new Staff {
                \new Voice {
                    {
                        \tempo 4=60
                        \time 4/4
                        c'4
                        cs'4
                        d'4
                        ef'4
                    }
                    {
                        e'4
                        f'4
                        fs'4
                        g'4
                    }
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    However, the behavior of the ``Quantizer`` can be modified at call-time.
    Passing a ``QSchema`` instance will alter the macro-structure of the
    output.

    Here, we quantize using settings specified by a ``MeasurewiseQSchema``,
    which will cause the ``Quantizer`` to group the output into measures
    with different tempi and time signatures:

    ::

        >>> measurewise_q_schema = quantizationtools.MeasurewiseQSchema(
        ...     {'tempo': ((1, 4), 78), 'time_signature': (2, 4)},
        ...     {'tempo': ((1, 8), 57), 'time_signature': (5, 4)},
        ...     )

    ::

        >>> result = quantizer(q_event_sequence, q_schema=measurewise_q_schema)
        >>> score = Score([Staff([result])])
        >>> print(format(score))
        \new Score <<
            \new Staff {
                \new Voice {
                    {
                        \tempo 4=78
                        \time 2/4
                        c'4 ~
                        \times 4/5 {
                            c'16.
                            cs'8.. ~
                        }
                    }
                    {
                        \tempo 8=57
                        \time 5/4
                        \times 4/7 {
                            cs'16.
                            d'8 ~
                        }
                        \times 4/5 {
                            d'16
                            ef'16. ~
                        }
                        \times 2/3 {
                            ef'16
                            e'8 ~
                        }
                        \times 4/7 {
                            e'16
                            f'8 ~
                            f'32 ~
                        }
                        f'32
                        fs'16. ~
                        \times 4/5 {
                            fs'32
                            g'8 ~
                        }
                        \times 4/7 {
                            g'32
                            r32
                            r16
                            r16
                            r16
                            r16
                            r16
                            r16
                        }
                        r4
                    }
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Here we quantize using settings specified by a ``BeatwiseQSchema``,
    which keeps the output of the quantizer "flattened", without measures or
    explicit time signatures.  The default beat-wise settings of quarter=60
    persists until the third "beatspan":

    ::

        >>> beatwise_q_schema = quantizationtools.BeatwiseQSchema(
        ... {
        ...     2: {'tempo': ((1, 4), 120)},
        ...     5: {'tempo': ((1, 4), 90)},
        ...     7: {'tempo': ((1, 4), 30)},
        ... })

    ::

        >>> result = quantizer(
        ...     q_event_sequence,
        ...     q_schema=beatwise_q_schema,
        ...     )
        >>> score = Score([Staff([result])])
        >>> print(format(score))
        \new Score <<
            \new Staff {
                \new Voice {
                    \tempo 4=60
                    c'4
                    cs'4
                    \tempo 4=120
                    d'2
                    ef'4 ~
                    \tempo 4=90
                    ef'8.
                    e'4 ~
                    e'16 ~
                    \times 2/3 {
                        \tempo 4=30
                        e'32
                        f'8.
                        fs'8 ~
                        fs'32 ~
                    }
                    \times 2/3 {
                        fs'32
                        g'8.
                        r32
                        r8
                    }
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Note that ``TieChains`` are generally fused together in the above example,
    but break at tempo changes.

    Other keyword arguments are:

        * ``grace_handler``: a ``GraceHandler`` instance controls whether and
          how grace notes are used in the output.  Options currently include
          ``CollapsingGraceHandler``, ``ConcatenatingGraceHandler`` and
          ``DiscardingGraceHandler``.

        * ``heuristic``: a ``Heuristic`` instance controls how output rhythms
          are selected from a pool of candidates.  Options currently include
          the ``DistanceHeuristic`` class.

        * ``job_handler``: a ``JobHandler`` instance controls whether or not
          parallel processing is used during the quantization process.
          Options include the ``SerialJobHandler`` and ``ParallelJobHandler``
          classes.

        * ``attack_point_optimizer``: an ``AttackPointOptimizer`` instance
          controls whether and how logical ties are re-notated.
          Options currently include ``MeasurewiseAttackPointOptimizer``,
          ``NaiveAttackPointOptimizer`` and ``NullAttackPointOptimizer``.

    Refer to the reference pages for ``BeatwiseQSchema`` and
    ``MeasurewiseQSchema`` for more information on controlling the
    ``Quantizer``'s output, and to the reference on ``SearchTree`` for
    information on controlling the rhythmic complexity of that same output.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self,
        q_event_sequence,
        q_schema=None,
        grace_handler=None,
        heuristic=None,
        job_handler=None,
        attack_point_optimizer=None,
        attach_tempos=True,
        ):
        r'''Calls quantizer.

        Returns Abjad components.
        '''
        from abjad.tools import quantizationtools

        q_event_sequence = quantizationtools.QEventSequence(q_event_sequence)

        if q_schema is None:
            q_schema = quantizationtools.MeasurewiseQSchema()
        assert isinstance(q_schema, quantizationtools.QSchema)

        q_target = q_schema(q_event_sequence.duration_in_ms)

        notation = q_target(q_event_sequence,
            grace_handler=grace_handler,
            heuristic=heuristic,
            job_handler=job_handler,
            attack_point_optimizer=attack_point_optimizer,
            attach_tempos=attach_tempos,
            )

        return notation



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import *


def test_lilypondparsertools_LilyPondParser__misc__variables_01():

    target = Staff([
        Container([
            Container([
                Container([
                    Container([Note(0, (1, 8))]),
                    Note(2, (1, 8)),
                    Note(4, (1, 4))
                ]),
                Note(5, (1, 4)),
                Note(7, (1, 2))
            ]),
            Note(9, (1, 2)),
            Note(11, 1)
        ]),
        Note(12, 1)
    ])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            {
                {
                    {
                        {
                            c'8
                        }
                        d'8
                        e'4
                    }
                    f'4
                    g'2
                }
                a'2
                b'1
            }
            c''1
        }
        '''
        )

    string = r'''
        foo = { c'8 }
        foo = { \foo d' e'4 }
        foo = { \foo f' g'2 }
        foo = { \foo a' b'1 }
        \new Staff { \foo c'' }
    '''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_RootedChordClass___init___01():
    r'''Initializes rooted chord-class from empty input.
    '''

    chord_class = tonalanalysistools.RootedChordClass()

    assert repr(chord_class) == 'CMajorTriadInRootPosition'


def test_tonalanalysistools_RootedChordClass___init___02():

    chord_class = tonalanalysistools.RootedChordClass(
        'g', 'dominant', 7, 'root')

    assert repr(chord_class) == 'GDominantSeventhInRootPosition'
    assert len(chord_class) == 4
    assert chord_class.root == pitchtools.NamedPitchClass('g')
    assert chord_class.bass == pitchtools.NamedPitchClass('g')


def test_tonalanalysistools_RootedChordClass___init___03():

    chord_class = tonalanalysistools.RootedChordClass(
        'g', 'dominant', 7, 'first')

    assert repr(chord_class) == 'GDominantSeventhInFirstInversion'
    assert len(chord_class) == 4
    assert chord_class.root == pitchtools.NamedPitchClass('g')
    assert chord_class.bass == pitchtools.NamedPitchClass('b')



# -*- coding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import override


class Beam(Spanner):
    r'''Beam.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8 g'2")
            >>> set_(staff).auto_beaming = False
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8
                d'8
                e'8
                f'8
                g'2
            }

        ::

            >>> beam = Beam()
            >>> attach(beam, staff[:2])
            >>> beam = Beam()
            >>> attach(beam, staff[2:4])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                c'8 [
                d'8 ]
                e'8 [
                f'8 ]
                g'2
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        direction=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        direction = stringtools.to_tridirectional_lilypond_symbol(
            direction)
        self._direction = direction

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        Spanner._copy_keyword_args(self, new)
        new._direction = self.direction

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        if self._is_my_first_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'override',
                is_once=False,
                )
            lilypond_format_bundle.grob_overrides.extend(contributions)
            if self.direction is not None:
                string = '{} ['.format(self.direction)
            else:
                string = '['
            lilypond_format_bundle.right.spanner_starts.append(string)
        if self._is_my_last_leaf(leaf):
            contributions = override(self)._list_format_contributions(
                'revert',
                )
            lilypond_format_bundle.grob_reverts.extend(contributions)
            string = ']'
            if self._is_my_first_leaf(leaf):
                lilypond_format_bundle.right.spanner_starts.append(string)
            else:
                lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC METHODS ###

    @staticmethod
    def _is_beamable(argument, beam_rests=False):
        '''Is true when `argument` is a beamable component. Otherwise false.

        ..  container:: example

            Without allowing for beamed rests:

            ::

                >>> staff = Staff(r"r32 a'32 ( [ gs'32 fs''32 \staccato f''8 ) ]")
                >>> staff.extend(r"r8 e''8 ( ef'2 )")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> for leaf in staff:
                ...     beam = spannertools.Beam
                ...     result = beam._is_beamable(leaf)
                ...     print('{:<8}\t{}'.format(leaf, result))
                ... 
                r32     False
                a'32    True
                gs'32   True
                fs''32  True
                f''8    True
                r8      False
                e''8    True
                ef'2    False

        ..  container:: example

            Allowing for beamed rests:

            ::

                >>> staff = Staff(r"r32 a'32 ( [ gs'32 fs''32 \staccato f''8 ) ]")
                >>> staff.extend(r"r8 e''8 ( ef'2 )")
                >>> show(staff) # doctest: +SKIP

            ::

                >>> for leaf in staff:
                ...     beam = spannertools.Beam
                ...     result = beam._is_beamable(
                ...         leaf,
                ...         beam_rests=True,
                ...         )
                ...     print('{:<8}\t{}'.format(leaf, result))
                ... 
                r32	True
                a'32	True
                gs'32	True
                fs''32	True
                f''8	True
                r8	True
                e''8	True
                ef'2	False

        ..  container:: example

            Is true for skips of any duration when `beam_rests` is true:

            ::

                >>> skip = Skip((1, 32))
                >>> Beam._is_beamable(skip, beam_rests=True)
                True

            ::

                >>> skip = Skip((1))
                >>> Beam._is_beamable(skip, beam_rests=True)
                True

        ..  container:: example

            Is true for rests of any duration when `beam_rests` is true:

            ::

                >>> rest = Rest((1, 32))
                >>> Beam._is_beamable(rest, beam_rests=True)
                True

            ::

                >>> rest = Rest((1))
                >>> Beam._is_beamable(rest, beam_rests=True)
                True

        Returns true or false.
        '''
        from abjad.tools import scoretools
        prototype = (scoretools.Note, scoretools.Chord)
        if isinstance(argument, prototype):
            if 0 < argument.written_duration.flag_count:
                return True
        prototype = (
            scoretools.MultimeasureRest,
            scoretools.Rest,
            scoretools.Skip,
            )
        if beam_rests and isinstance(argument, prototype):
            return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets direction.

        Defaults to none.

        Set to up, down or none.

        Returns up, down or none.
        '''
        return self._direction



# -*- coding: utf-8 -*-
import json
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_1(self, open_file_mock):
        self.create_score()
        self.install_fancy_segment_maker()
        path_1 = self.create_segment('segment_one')
        path_2 = self.create_segment('segment_two')
        path_3 = self.create_segment('segment_three')
        self.illustrate_segments()
        with open(str(path_1.joinpath('metadata.json')), 'r') as file_pointer:
            metadata_1 = json.loads(file_pointer.read())
        with open(str(path_2.joinpath('metadata.json')), 'r') as file_pointer:
            metadata_2 = json.loads(file_pointer.read())
        with open(str(path_3.joinpath('metadata.json')), 'r') as file_pointer:
            metadata_3 = json.loads(file_pointer.read())
        assert metadata_1 == {
            'first_bar_number': 1,
            'measure_count': 1,
            'segment_count': 3,
            'segment_number': 1,
            }
        assert metadata_2 == {
            'first_bar_number': 2,
            'measure_count': 1,
            'segment_count': 3,
            'segment_number': 2,
            }
        assert metadata_3 == {
            'first_bar_number': 3,
            'measure_count': 1,
            'segment_count': 3,
            'segment_number': 3,
            }

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_2(self, open_file_mock):
        self.create_score()
        self.install_fancy_segment_maker()
        segment_path = self.create_segment('test_segment')
        self.illustrate_segment('test_segment')
        illustration_path = segment_path.joinpath('illustration.ly')
        self.compare_lilypond_contents(
            illustration_path,
            r'''
            \language "english"
            <BLANKLINE>
            \include "../../stylesheets/stylesheet.ily"
            <BLANKLINE>
            \header {}
            <BLANKLINE>
            \layout {}
            <BLANKLINE>
            \paper {}
            <BLANKLINE>
            \score {
                \context Score = "String Quartet Score" <<
                    \context StaffGroup = "String Quartet Staff Group" <<
                        \tag #'first-violin
                        \context Staff = "First Violin Staff" {
                            \clef "treble"
                            \set Staff.instrumentName = \markup { Violin }
                            \set Staff.shortInstrumentName = \markup { Vn. }
                            \context Voice = "First Violin Voice" {
                                {
                                    \time 4/4
                                    c'1
                                    \bar "|."
                                }
                            }
                        }
                        \tag #'second-violin
                        \context Staff = "Second Violin Staff" {
                            \clef "treble"
                            \set Staff.instrumentName = \markup { Violin }
                            \set Staff.shortInstrumentName = \markup { Vn. }
                            \context Voice = "Second Violin Voice" {
                                {
                                    \time 4/4
                                    c'1
                                    \bar "|."
                                }
                            }
                        }
                        \tag #'viola
                        \context Staff = "Viola Staff" {
                            \clef "alto"
                            \set Staff.instrumentName = \markup { Viola }
                            \set Staff.shortInstrumentName = \markup { Va. }
                            \context Voice = "Viola Voice" {
                                {
                                    \time 4/4
                                    c'1
                                    \bar "|."
                                }
                            }
                        }
                        \tag #'cello
                        \context Staff = "Cello Staff" {
                            \clef "bass"
                            \set Staff.instrumentName = \markup { Cello }
                            \set Staff.shortInstrumentName = \markup { Vc. }
                            \context Voice = "Cello Voice" {
                                {
                                    \time 4/4
                                    c'1
                                    \bar "|."
                                }
                            }
                        }
                    >>
                >>
            }
            '''
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet__fix_01():
    r'''Halve note durations.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'4 d'4 e'4")
    assert not tuplet.multiplier.is_proper_tuplet_multiplier

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 1/3 {
            c'4
            d'4
            e'4
        }
        '''
        )

    tuplet._fix()

    assert format(tuplet) == stringtools.normalize(
        r'''
        \times 2/3 {
            c'8
            d'8
            e'8
        }
        '''
        )

    assert tuplet.multiplier.is_proper_tuplet_multiplier
    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Tuplet__fix_02():
    r'''Double note duration.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'32 d'32 e'32")
    assert not tuplet.multiplier.is_proper_tuplet_multiplier

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 8/3 {
            c'32
            d'32
            e'32
        }
        '''
        )

    tuplet._fix()

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 4/3 {
            c'16
            d'16
            e'16
        }
        '''
        )

    assert tuplet.multiplier.is_proper_tuplet_multiplier
    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Tuplet__fix_03():
    r'''Halve note durations.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(5, 16), "c'4 d'4 e'4")
    assert not tuplet.multiplier.is_proper_tuplet_multiplier

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 5/12 {
            c'4
            d'4
            e'4
        }
        '''
        )

    tuplet._fix()

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 5/6 {
            c'8
            d'8
            e'8
        }
        '''
        )

    assert tuplet.multiplier.is_proper_tuplet_multiplier
    assert inspect_(tuplet).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import mutate


def edit_cello_voice(score, durated_reservoir):
    r'''Edits cello voice.
    '''

    voice = score['Cello Voice']
    descents = durated_reservoir['Cello']

    logical_tie = inspect_(voice[-1]).get_logical_tie()
    for leaf in logical_tie.leaves:
        parent = leaf._get_parentage().parent
        index = parent.index(leaf)
        parent[index] = scoretools.Chord(['e,', 'a,'], leaf.written_duration)

    selection = voice[-len(descents[-1]):]
    unison_descent = mutate(selection).copy()
    voice.extend(unison_descent)
    for chord in unison_descent:
        index = inspect_(chord).get_parentage().parent.index(chord)
        parent[index] = scoretools.Note(
            chord.written_pitches[1], chord.written_duration)
        articulation = indicatortools.Articulation('accent')
        attach(articulation, parent[index])
        articulation = indicatortools.Articulation('tenuto')
        attach(articulation, parent[index])

    voice.extend('a,1. ~ a,2')
    voice.extend('b,1 ~ b,1. ~ b,1.')
    voice.extend('a,1. ~ a,1. ~ a,1. ~ a,1. ~ a,1. ~ a,2')
    voice.extend('r4 r2.')


# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class LilyPondFile(AbjadObject):
    r'''A LilyPond file.

    ..  container:: example

        Makes LilyPond file:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> comments = [
            ...     'File construct as an example.',
            ...     'Parts shown here for positioning.',
            ...     ]
            >>> includes = [
            ...     'external-settings-file-1.ly',
            ...     'external-settings-file-2.ly',
            ...     ]
            >>> lilypond_file = LilyPondFile.new(
            ...     music=staff,
            ...     default_paper_size=('a5', 'portrait'),
            ...     comments=comments,
            ...     includes=includes,
            ...     global_staff_size=16,
            ...     )

        ::

            >>> lilypond_file.header_block.composer = Markup('Josquin')
            >>> lilypond_file.header_block.title = Markup('Missa sexti tonus')
            >>> lilypond_file.layout_block.indent = 0
            >>> lilypond_file.layout_block.left_margin = 15

        ::

            >>> lilypond_file
            LilyPondFile(comments=['File construct as an example.', 'Parts
            shown here for positioning.'],
            date_time_token=DateTimeToken(date_string='...'),
            default_paper_size=('a5', 'portrait'), global_staff_size=16,
            includes=['external-settings-file-1.ly',
            'external-settings-file-2.ly'], items=[<Block(name='header')>,
            <Block(name='layout')>, <Block(name='paper')>,
            <Block(name='score')>],
            lilypond_language_token=LilyPondLanguageToken(),
            lilypond_version_token=LilyPondVersionToken(version_string='...'))

        ::

            >>> print(format(lilypond_file)) # doctest: +SKIP
            % 2004-01-14 17:29

            % File construct as an example.
            % Parts shown here for positioning.

            \version "2.19.0"
            \language "english"

            \include "external-settings-file-1.ly"
            \include "external-settings-file-2.ly"

            #(set-default-paper-size "a5" 'portrait)
            #(set-global-staff-size 16)

            \header {
                composer = \markup { Josquin }
                title = \markup { Missa sexti tonus }
            }

            \layout {
                indent = #0
                left-margin = #15
            }

            \paper {
            }

            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }

        ::

            >>> show(lilypond_file) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_comments',
        '_date_time_token',
        '_default_paper_size',
        '_global_staff_size',
        '_includes',
        '_items',
        '_lilypond_language_token',
        '_lilypond_version_token',
        '_use_relative_includes',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        comments=None,
        date_time_token=None,
        default_paper_size=None,
        global_staff_size=None,
        includes=None,
        items=None,
        lilypond_language_token=None,
        lilypond_version_token=None,
        use_relative_includes=None,
        ):
        from abjad.tools import lilypondfiletools
        comments = comments or ()
        comments = tuple(comments)
        self._comments = comments
        self._date_time_token = None
        if date_time_token is not False:
            self._date_time_token = lilypondfiletools.DateTimeToken()
        self._default_paper_size = default_paper_size
        self._global_staff_size = global_staff_size
        includes = includes or ()
        includes = tuple(includes)
        self._includes = includes
        self._items = items or []
        self._lilypond_language_token = None
        if lilypond_language_token is not False:
            token = lilypondfiletools.LilyPondLanguageToken()
            self._lilypond_language_token = token
        self._lilypond_version_token = None
        if lilypond_version_token is not False:
            token = lilypondfiletools.LilyPondVersionToken()
            self._lilypond_version_token = token
        self._use_relative_includes = use_relative_includes

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond file.

        ..  container:: example

            Gets format:

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> print(format(lilypond_file)) # doctest: +SKIP
                % 2016-01-31 20:29
                <BLANKLINE>
                \version "2.19.35"
                \language "english"
                <BLANKLINE>
                \header {}
                <BLANKLINE>
                \layout {}
                <BLANKLINE>
                \paper {}

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getitem__(self, name):
        r'''Gets item with `name`.

        ..  container:: example

            Gets header block:

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file['header']
                <Block(name='header')>

        ..  container:: example

            Searches score:

            ::

                >>> voice_1 = Voice("c''4 b' a' g'", name='Custom Voice 1')
                >>> attach(LilyPondCommand('voiceOne'), voice_1)
                >>> voice_2 = Voice("c'4 d' e' f'", name='Custom Voice 2')
                >>> attach(LilyPondCommand('voiceTwo'), voice_2)
                >>> staff = Staff(
                ...     [voice_1, voice_2],
                ...     is_simultaneous=True,
                ...     name='Custom Staff',
                ...     )
                >>> score = Score([staff], name='Custom Score')
                >>> lilypond_file = lilypondfiletools.LilyPondFile.new(score)
                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \context Score = "Custom Score" <<
                    \context Staff = "Custom Staff" <<
                        \context Voice = "Custom Voice 1" {
                            \voiceOne
                            c''4
                            b'4
                            a'4
                            g'4
                        }
                        \context Voice = "Custom Voice 2" {
                            \voiceTwo
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                    >>
                >>
                
            ::

                >>> lilypond_file['score']
                <Block(name='score')>

            ::

                >>> lilypond_file['Custom Score']
                <Score-"Custom Score"<<1>>>

            ::

                >>> lilypond_file[Score]
                <Score-"Custom Score"<<1>>>

            ::

                >>> lilypond_file['Custom Staff']
                <Staff-"Custom Staff"<<2>>>

            ::

                >>> lilypond_file[Staff]
                <Staff-"Custom Staff"<<2>>>

            ::

                >>> lilypond_file['Custom Voice 1']
                Voice("c''4 b'4 a'4 g'4")

            ::

                >>> lilypond_file['Custom Voice 2']
                Voice("c'4 d'4 e'4 f'4")

            ::

                >>> lilypond_file[Voice]
                Voice("c''4 b'4 a'4 g'4")

        Returns item.

        Raises key error when no item with `name` is found.
        '''
        import abjad
        if self.score_block and self.score_block.items:
            score = self.score_block.items[0]
        else:
            score = None
        if isinstance(name, str):
            for item in self.items:
                if getattr(item, 'name', None) == name:
                    return item
            if score is not None:
                if score.name == name:
                    return score
                context = score[name]
                return context
            message = 'can not find item with name: {!r}.'
            message = message.format(name)
            raise KeyError(message)
        else:
            for item in self.items:
                if isinstance(item, name):
                    return item
            if score is not None:
                if isinstance(score, name):
                    return score
                prototype = abjad.scoretools.Context
                for context in abjad.iterate(score).by_class(prototype):
                    if isinstance(context, name):
                        return context
            message = 'can not find item of class: {!r}.'
            message = message.format(name)
            raise KeyError(message)

    def __illustrate__(self):
        r'''Illustrates LilyPond file.

        Returns LilyPond file unchanged.
        '''
        return self

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond file.

        ..  container:: example

            Gets interpreter representation:

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file
                LilyPondFile(comments=[],
                date_time_token=DateTimeToken(date_string='...'), includes=[],
                items=[<Block(name='header')>, <Block(name='layout')>,
                <Block(name='paper')>, <Block(name='score')>],
                lilypond_language_token=LilyPondLanguageToken(),
                lilypond_version_token=LilyPondVersionToken(version_string='...'))

        Returns string.
        '''
        superclass = super(LilyPondFile, self)
        return superclass.__repr__()

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        result = []
        if self.date_time_token is not None:
            string = '% {}'.format(self.date_time_token)
            result.append(string)
        result.extend(self._formatted_comments)
        includes = []
        if self.lilypond_version_token is not None:
            string = '{}'.format(self.lilypond_version_token)
            includes.append(string)
        if self.lilypond_language_token is not None:
            string = '{}'.format(self.lilypond_language_token)
            includes.append(string)
        includes = '\n'.join(includes)
        if includes:
            result.append(includes)
        if self.use_relative_includes:
            string = "#(ly:set-option 'relative-includes #t)"
            result.append(string)
        result.extend(self._formatted_includes)
        result.extend(self._formatted_scheme_settings)
        result.extend(self._formatted_blocks)
        return result

    ### PUBLIC METHODS ###

    @classmethod
    def new(
        cls,
        music=None,
        date_time_token=None,
        default_paper_size=None,
        comments=None,
        includes=None,
        global_staff_size=None,
        lilypond_language_token=None,
        lilypond_version_token=None,
        use_relative_includes=None,
        ):
        r'''Makes basic LilyPond file.

        ..  container:: example

            **Example 1.** Makes basic LilyPond file:

            ::

                >>> score = Score([Staff("c'8 d'8 e'8 f'8")])
                >>> lilypond_file = LilyPondFile.new(score)
                >>> lilypond_file.header_block.title = Markup('Missa sexti tonus')
                >>> lilypond_file.header_block.composer = Markup('Josquin')
                >>> lilypond_file.layout_block.indent = 0
                >>> lilypond_file.paper_block.top_margin = 15
                >>> lilypond_file.paper_block.left_margin = 15

            ::

                >>> print(format(lilypond_file)) # doctest: +SKIP
                \header {
                    composer = \markup { Josquin }
                    title = \markup { Missa sexti tonus }
                }

                \layout {
                    indent = #0
                }

                \paper {
                    left-margin = #15
                    top-margin = #15
                }

                \score {
                    \new Score <<
                        \new Staff {
                            c'8
                            d'8
                            e'8
                            f'8
                        }
                    >>
                }

            ::

                >>> show(lilypond_file) # doctest: +SKIP

        Wraps `music` in LilyPond ``\score`` block.

        Adds LilyPond ``\header``, ``\layout``, ``\paper`` and ``\score``
        blocks to LilyPond file.

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        if isinstance(music, lilypondfiletools.LilyPondFile):
            return music
        lilypond_file = cls(
            date_time_token=date_time_token,
            default_paper_size=default_paper_size,
            comments=comments,
            includes=includes,
            items=[
                lilypondfiletools.Block(name='header'),
                lilypondfiletools.Block(name='layout'),
                lilypondfiletools.Block(name='paper'),
                lilypondfiletools.Block(name='score'),
                ],
            global_staff_size=global_staff_size,
            lilypond_language_token=lilypond_language_token,
            lilypond_version_token=lilypond_version_token,
            use_relative_includes=use_relative_includes,
            )
        if music is not None:
            lilypond_file.score_block.items.append(music)
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_blocks(self):
        result = []
        for x in self.items:
            if '_get_lilypond_format' in dir(x) and not isinstance(x, str):
                lilypond_format = format(x)
                if lilypond_format:
                    result.append(lilypond_format)
            else:
                result.append(str(x))
        return result

    @property
    def _formatted_comments(self):
        result = []
        for comment in self.comments:
            if ('_get_lilypond_format' in dir(comment) and
                not isinstance(comment, str)):
                lilypond_format = format(comment)
                if lilypond_format:
                    string = '% {}'.format(comment)
                    result.append(string)
            else:
                string = '% {!s}'.format(comment)
                result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_includes(self):
        result = []
        for include in self.includes:
            if isinstance(include, str):
                string = r'\include "{}"'.format(include)
                result.append(string)
            else:
                result.append(format(include))
        if result:
            result = ['\n'.join(result)]
        return result

    @property
    def _formatted_scheme_settings(self):
        result = []
        default_paper_size = self.default_paper_size
        if default_paper_size is not None:
            dimension, orientation = default_paper_size
            string = "#(set-default-paper-size \"{}\" '{})"
            string = string.format(dimension, orientation)
            result.append(string)
        global_staff_size = self.global_staff_size
        if global_staff_size is not None:
            string = '#(set-global-staff-size {})'
            string = string.format(global_staff_size)
            result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        result = []
        if self.date_time_token is not None:
            string = '% {}'.format(self.date_time_token)
            result.append(string)
        result.extend(self._formatted_comments)
        includes = []
        if self.lilypond_version_token is not None:
            string = '{}'.format(self.lilypond_version_token)
            includes.append(string)
        if self.lilypond_language_token is not None:
            string = '{}'.format(self.lilypond_language_token)
            includes.append(string)
        includes = '\n'.join(includes)
        if includes:
            result.append(includes)
        if self.use_relative_includes:
            string = "#(ly:set-option 'relative-includes #t)"
            result.append(string)
        result.extend(self._formatted_includes)
        result.extend(self._formatted_scheme_settings)
        result.extend(self._formatted_blocks)
        return result

    def _get_lilypond_format(self):
        return '\n\n'.join(self._get_format_pieces())

    ### PUBLIC PROPERTIES ###

    @property
    def comments(self):
        r'''Gets comments of Lilypond file.

        ..  container:: example

            Gets comments:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.comments
                []

        Returns list.
        '''
        return list(self._comments)

    @property
    def date_time_token(self):
        r'''Gets date-time token.

        ..  container:: example

            Gets date-time token:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.date_time_token
                DateTimeToken()

        Returns date-time token or none.
        '''
        return self._date_time_token

    @property
    def default_paper_size(self):
        r'''Gets default paper size of LilyPond file.

        ..  container:: example

            Gets default paper size:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.default_paper_size is None
                True

        Set to pair or none.

        Defaults to none.

        Returns pair or none.
        '''
        return self._default_paper_size

    @property
    def global_staff_size(self):
        r'''Gets global staff size of LilyPond file.

        ..  container:: example

            Gets global staff size:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.global_staff_size is None
                True

        Set to number or none.

        Defaults to none.

        Returns number or none.
        '''
        return self._global_staff_size

    @property
    def header_block(self):
        r'''Gets header block.

        ..  container:: example

            Gets header block:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.header_block
                <Block(name='header')>

        Returns block or none.
        '''
        from abjad.tools import lilypondfiletools
        for item in self.items:
            if isinstance(item, lilypondfiletools.Block):
                if item.name == 'header':
                    return item

    @property
    def includes(self):
        r'''Gets includes of LilyPond file.

        ..  container:: example

            Gets includes:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.includes
                []

        Returns list.
        '''
        return list(self._includes)

    @property
    def items(self):
        r'''Gets items in LilyPond file.

        ..  container:: example

            Gets items:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> for item in lilypond_file.items:
                ...     item
                ...
                <Block(name='header')>
                <Block(name='layout')>
                <Block(name='paper')>
                <Block(name='score')>

        ..  container:: example

            Returns list:

            >>> isinstance(lilypond_file.items, list)
            True

        Returns list.
        '''
        return self._items

    @property
    def layout_block(self):
        r'''Gets layout block.

        ..  container:: example

            Gets layout block:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.layout_block
                <Block(name='layout')>

        Returns block or none.
        '''
        from abjad.tools import lilypondfiletools
        for item in self.items:
            if isinstance(item, lilypondfiletools.Block):
                if item.name == 'layout':
                    return item

    @property
    def lilypond_language_token(self):
        r'''Gets LilyPond language token.

        ..  container:: example

            Gets LilyPond language token:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.lilypond_language_token
                LilyPondLanguageToken()

        Returns LilyPond language token or none.
        '''
        return self._lilypond_language_token

    @property
    def lilypond_version_token(self):
        r'''Gets LilyPond version token.

        ..  container:: example

            Gets LilyPond version token:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.lilypond_version_token # doctest: +SKIP
                LilyPondVersionToken('2.19.35')

        Returns LilyPond version token or none.
        '''
        return self._lilypond_version_token

    @property
    def paper_block(self):
        r'''Gets paper block.

        ..  container:: example

            Gets paper block:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.paper_block
                <Block(name='paper')>

        Returns block or none.
        '''
        from abjad.tools import lilypondfiletools
        for item in self.items:
            if isinstance(item, lilypondfiletools.Block):
                if item.name == 'paper':
                    return item

    @property
    def score_block(self):
        r'''Gets score block.

        ..  container:: example

            Gets score block:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.score_block
                <Block(name='score')>

        Returns block or none.
        '''
        from abjad.tools import lilypondfiletools
        for item in self.items:
            if isinstance(item, lilypondfiletools.Block):
                if item.name == 'score':
                    return item

    @property
    def use_relative_includes(self):
        r'''Is true when LilyPond file should use relative includes.

        ..  container:: example

            Gets relative include flag:

            ::

                >>> lilypond_file = LilyPondFile.new()

            ::

                >>> lilypond_file.use_relative_includes is None
                True

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._use_relative_includes



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import systemtools


def test_systemtools_IOManager_get_next_output_file_name_01():

    next_output_file_name = systemtools.IOManager.get_next_output_file_name()

    assert isinstance(next_output_file_name, str)
    assert len(next_output_file_name) == 7
    assert next_output_file_name.endswith('.ly')



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class RehearsalMark(AbjadValueObject):
    r'''Rehearsal mark.

    ..  container:: example

        **Example 1.** Rehearsal A:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> score = Score([staff])
            >>> mark = indicatortools.RehearsalMark(number=1)
            >>> attach(mark, staff[0])
            >>> scheme = schemetools.Scheme('format-mark-box-alphabet')
            >>> set_(score).markFormatter = scheme
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score \with {
                markFormatter = #format-mark-box-alphabet
            } <<
                \new Staff {
                    \mark #1
                    c'4
                    d'4
                    e'4
                    f'4
                }
            >>

    ..  container:: example

        **Example 2.** Rehearsal B:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> score = Score([staff])
            >>> mark = indicatortools.RehearsalMark(number=2)
            >>> attach(mark, staff[0])
            >>> scheme = schemetools.Scheme('format-mark-box-alphabet')
            >>> set_(score).markFormatter = scheme
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score \with {
                markFormatter = #format-mark-box-alphabet
            } <<
                \new Staff {
                    \mark #2
                    c'4
                    d'4
                    e'4
                    f'4
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_markup',
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, number=None, markup=None):
        from abjad.tools import markuptools
        # TODO: make default scope work
        #self._default_scope = scoretools.Score
        self._default_scope = None
        if markup is not None:
            assert isinstance(markup, markuptools.Markup)
        self._markup = markup
        if number is not None:
            assert mathtools.is_positive_integer(number)
        self._number = number

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of rehearsal mark.

        ..  container:: example

            **Example 1.** Rehearsal A

            ::

                >>> mark = indicatortools.RehearsalMark(number=1)
                >>> print(str(mark))
                \mark #1

        ..  container:: example

            **Example 2.** Rehearsal B

            ::

                >>> mark = indicatortools.RehearsalMark(number=2)
                >>> print(str(mark))
                \mark #2

        Returns string.
        '''
        return self._get_lilypond_format()

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        if self.markup is not None:
            result = r'\mark {}'.format(self.markup)
        elif self.number is not None:
            result = r'\mark #{}'.format(self.number)
        else:
            result = r'\mark \default'
        return result

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.opening.commands.append(str(self))
        return lilypond_format_bundle
        
    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of rehearsal mark.

        ..  container:: example

            **Example 1.** Rehearsal A:

            ::

                >>> mark = indicatortools.RehearsalMark(number=1)
                >>> mark.default_scope is None
                True

        ..  container:: example

            **Example 2.** Rehearsal B:

            ::

                >>> mark = indicatortools.RehearsalMark(number=2)
                >>> mark.default_scope is None
                True

        ..  todo:: Make rehearsal marks score-scoped.

        Returns none (but should return score).
        '''
        return self._default_scope

    @property
    def markup(self):
        r'''Gets markup of rehearsal mark.

        ..  container:: example

            **Example 1.** Custom rehearsal A:

            ::

                >>> markup = Markup(r'\bold { \italic { A } }')
                >>> mark = indicatortools.RehearsalMark(markup=markup)
                >>> show(mark.markup) # doctest: +SKIP

            ..  doctest::

                >>> print(str(mark.markup))
                \markup {
                    \bold
                        {
                            \italic
                                {
                                    A
                                }
                        }
                    }

        ..  container:: example

            **Example 2.** Custom rehearsal B:

            ::

                >>> markup = Markup(r'\bold { \italic { B } }')
                >>> mark = indicatortools.RehearsalMark(markup=markup)
                >>> show(mark.markup) # doctest: +SKIP

            ..  doctest::

                >>> print(str(mark.markup))
                \markup {
                    \bold
                        {
                            \italic
                                {
                                    B
                                }
                        }
                    }

        Returns markup or none.
        '''
        return self._markup

    @property
    def number(self):
        r'''Gets number of rehearsal mark.

        ..  container:: example

            **Example 1.** Rehearsal A:

            ::

                >>> mark = indicatortools.RehearsalMark(number=1)
                >>> mark.number
                1

        ..  container:: example

            **Example 2.** Rehearsal B:

            ::

                >>> mark = indicatortools.RehearsalMark(number=2)
                >>> mark.number
                2

        Returns positive integer or none.
        '''
        return self._number



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_timeline_01():
    r'''Yields component class instances in argument sorted backward
    by score offset stop time.
    '''

    score = Score()
    staff_1 = Staff("c'4 d'4 e'4 f'4")
    staff_2 = Staff("g'8 a'8 b'8 c''8")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        >>
        '''
        )

    leaf_generator = iterate(score).by_timeline(reverse=True)
    leaves = list(leaf_generator)

    assert leaves[0] is staff_1[3] # f'4
    assert leaves[1] is staff_1[2] # e'4
    assert leaves[2] is staff_1[1] # d'4
    assert leaves[3] is staff_2[3] # c''8
    assert leaves[4] is staff_2[2] # b'8
    assert leaves[5] is staff_1[0] # c'4
    assert leaves[6] is staff_2[1] # a'8
    assert leaves[7] is staff_2[0] # g'8


def test_agenttools_IterationAgent_by_timeline_02():
    r'''Yields component class instances in argument sorted by score offset
    and score index.
    '''

    score = Score()
    staff_1 = Staff("c'8 d'8 e'8 f'8")
    staff_2 = Staff("g'4 a'4 b'4 c''4")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                g'4
                a'4
                b'4
                c''4
            }
        >>
        '''
        )

    leaf_generator = iterate(score).by_timeline(reverse=True)
    leaves = list(leaf_generator)

    assert leaves[0] is staff_2[3] # c''4
    assert leaves[1] is staff_2[2] # b'4
    assert leaves[2] is staff_1[3] # f'8
    assert leaves[3] is staff_2[1] # a'4
    assert leaves[4] is staff_1[2] # e'8
    assert leaves[5] is staff_1[1] # d'8
    assert leaves[6] is staff_2[0] # g'4
    assert leaves[7] is staff_1[0] # c'8


def test_agenttools_IterationAgent_by_timeline_03():
    r'''Yields component class instances in argument sorted by score offset
    and score index.
    '''

    score = Score()
    staff_1 = Staff("c'4 d'4 e'4 f'4")
    staff_2 = Staff("g'8 a'8 b'8 c''8")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
            }
            \new Staff {
                g'8
                a'8
                b'8
                c''8
            }
        >>
        '''
        )

    leaf_generator = iterate(score).by_timeline()
    leaves = list(leaf_generator)

    assert leaves[0] is staff_1[0] # c'4
    assert leaves[1] is staff_2[0] # g'8
    assert leaves[2] is staff_2[1] # a'8
    assert leaves[3] is staff_1[1] # d'4
    assert leaves[4] is staff_2[2] # b'8
    assert leaves[5] is staff_2[3] # c''8
    assert leaves[6] is staff_1[2] # e'4
    assert leaves[7] is staff_1[3] # f'4


def test_agenttools_IterationAgent_by_timeline_04():
    r'''Yields component class instances in argument sorted by score offset
    and score index.
    '''

    score = Score()
    staff_1 = Staff("c'8 d'8 e'8 f'8")
    staff_2 = Staff("g'4 a'4 b'4 c''4")
    score.extend([staff_1, staff_2])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                g'4
                a'4
                b'4
                c''4
            }
        >>
        '''
        )

    leaf_generator = iterate(score).by_timeline()
    leaves = list(leaf_generator)

    assert leaves[0] is staff_1[0] # c'8
    assert leaves[1] is staff_2[0] # g'4
    assert leaves[2] is staff_1[1] # d'8
    assert leaves[3] is staff_1[2] # e'8
    assert leaves[4] is staff_2[1] # a'4
    assert leaves[5] is staff_1[3] # f'8
    assert leaves[6] is staff_2[2] # b'4
    assert leaves[7] is staff_2[3] # c''4


def test_agenttools_IterationAgent_by_timeline_05():

    staff_one = Staff(r"c'4 \times 2/3 { d'8 e' f' } g'2")
    staff_two = Staff('r2 fs2')
    staff_group = StaffGroup([staff_one, staff_two])
    score = Score([staff_group])

    component_generator = iterate(score).by_timeline(scoretools.Component)
    components = tuple(component_generator)

    assert components[0] is score
    assert components[1] is staff_group
    assert components[2] is staff_one
    assert components[3] is staff_one[0]
    assert components[4] is staff_two
    assert components[5] is staff_two[0]
    assert components[6] is staff_one[1]
    assert components[7] is staff_one[1][0]
    assert components[8] is staff_one[1][1]
    assert components[9] is staff_one[1][2]
    assert components[10] is staff_one[2]
    assert components[11] is staff_two[1]


def test_agenttools_IterationAgent_by_timeline_06():

    staff_one = Staff(r"c'4 \times 2/3 { d'8 e' f' } g'2")
    staff_two = Staff('r2 fs2')
    staff_group = StaffGroup([staff_one, staff_two])
    score = Score([staff_group])

    component_generator = iterate(score).by_timeline(
        scoretools.Component,
        reverse=True,
        )
    components = tuple(component_generator)

    assert components[0] is score
    assert components[1] is staff_group
    assert components[2] is staff_one
    assert components[3] is staff_one[2]
    assert components[4] is staff_two
    assert components[5] is staff_two[1]
    assert components[6] is staff_one[1]
    assert components[7] is staff_one[1][2]
    assert components[8] is staff_two[0]
    assert components[9] is staff_one[1][1]
    assert components[10] is staff_one[1][0]
    assert components[11] is staff_one[0]


def test_agenttools_IterationAgent_by_timeline_07():

    staff_one = Staff(r"c'4 << \times 2/3 { d' e' f' } { b' a' } >> g'4")
    staff_two = Staff("c,2 e,2")
    staff_group = StaffGroup([staff_one, staff_two])
    score = Score([staff_group])

    component_generator = iterate(score).by_timeline(
        scoretools.Component,
        )
    components = tuple(component_generator)

    assert components[0] is score
    assert components[1] is staff_group
    assert components[2] is staff_one
    assert components[3] is staff_one[0]
    assert components[4] is staff_two
    assert components[5] is staff_two[0]
    assert components[6] is staff_one[1]
    assert components[7] is staff_one[1][0]
    assert components[8] is staff_one[1][0][0]
    assert components[9] is staff_one[1][1]
    assert components[10] is staff_one[1][1][0]
    assert components[11] is staff_one[1][0][1]
    assert components[12] is staff_one[1][1][1]
    assert components[13] is staff_two[1]
    assert components[14] is staff_one[1][0][2]
    assert components[15] is staff_one[2]


def test_agenttools_IterationAgent_by_timeline_08():

    staff_one = Staff(r"c'4 << \times 2/3 { d' e' f' } { b' a' } >> g'4")
    staff_two = Staff("c,2 e,2")
    staff_group = StaffGroup([staff_one, staff_two])
    score = Score([staff_group])

    component_generator = iterate(score).by_timeline(
        scoretools.Component,
        reverse=True
        )
    components = tuple(component_generator)

    assert components[0] is score
    assert components[1] is staff_group
    assert components[2] is staff_one
    assert components[3] is staff_one[2]
    assert components[4] is staff_two
    assert components[5] is staff_two[1]
    assert components[6] is staff_one[1]
    assert components[7] is staff_one[1][0]
    assert components[8] is staff_one[1][0][2]
    assert components[9] is staff_one[1][1]
    assert components[10] is staff_one[1][1][1]
    assert components[11] is staff_one[1][0][1]
    assert components[12] is staff_one[1][1][0]
    assert components[13] is staff_two[0]
    assert components[14] is staff_one[1][0][0]
    assert components[15] is staff_one[0]



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchSegment___slots___01():

    ncps = ['bf', 'bqf', "fs'", "g'", 'bqf', "g'"]
    named_pitch_segment = pitchtools.PitchSegment(ncps)

    assert pytest.raises(AttributeError, "named_pitch_segment.foo = 'bar'")



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Chord___copy___01():

    chord_1 = Chord("<ef' cs'' f''>4")
    chord_2 = copy.copy(chord_1)

    assert isinstance(chord_1, Chord)
    assert isinstance(chord_2, Chord)
    assert format(chord_1) == format(chord_2)
    assert chord_1 is not chord_2


def test_scoretools_Chord___copy___02():
    r'''Chord copies LilyPond duration multiplier.
    '''

    chord_1 = Chord("<ef' cs'' f''>4 * 1/2")
    chord_2 = copy.copy(chord_1)

    assert isinstance(chord_1, Chord)
    assert isinstance(chord_2, Chord)
    assert format(chord_1) == format(chord_2)
    assert chord_1 is not chord_2


def test_scoretools_Chord___copy___03():
    r'''Chord copies LilyPond grob overrides and LilyPond context settings.
    '''

    chord_1 = Chord("<ef' cs'' f''>4")
    override(chord_1).staff.note_head.color = 'red'
    override(chord_1).accidental.color = 'red'
    set_(chord_1).tuplet_full_length = True
    chord_2 = copy.copy(chord_1)

    assert isinstance(chord_1, Chord)
    assert isinstance(chord_2, Chord)
    assert format(chord_1) == format(chord_2)
    assert chord_1 is not chord_2


def test_scoretools_Chord___copy___04():
    r'''Chord copies tweaked note-heads.
    '''

    chord_1 = Chord("<c' e' g'>4")
    chord_1.note_heads[0].tweak.color = 'red'
    chord_2 = copy.copy(chord_1)

    assert format(chord_1) == stringtools.normalize(
        r'''
        <
            \tweak color #red
            c'
            e'
            g'
        >4
        '''
        )

    assert format(chord_2) == stringtools.normalize(
        r'''
        <
            \tweak color #red
            c'
            e'
            g'
        >4
        '''
        )

    assert chord_2.note_heads[0]._client is chord_2
    assert chord_2.note_heads[1]._client is chord_2
    assert chord_2.note_heads[2]._client is chord_2

    assert format(chord_1) == format(chord_2)
    assert chord_1 is not chord_2

    assert chord_1.note_heads[0] == chord_2.note_heads[0]
    assert chord_1.note_heads[1] == chord_2.note_heads[1]
    assert chord_1.note_heads[2] == chord_2.note_heads[2]

    assert chord_1.note_heads[0] is not chord_2.note_heads[0]
    assert chord_1.note_heads[1] is not chord_2.note_heads[1]
    assert chord_1.note_heads[2] is not chord_2.note_heads[2]


def test_scoretools_Chord___copy___05():
    r'''Chord coipes articulations and markup.
    '''

    chord_1 = Chord("<ef' cs'' f''>4")
    articulation_1 = Articulation('staccato')
    attach(articulation_1, chord_1)
    markup_1 = markuptools.Markup('foo', Up)
    attach(markup_1, chord_1)

    chord_2 = copy.copy(chord_1)

    assert format(chord_1) == format(chord_2)
    assert chord_1 is not chord_2

    articulation_2 = inspect_(chord_2).get_indicators(Articulation)[0]
    assert articulation_1 == articulation_2
    assert articulation_1 is not articulation_2

    markup_2 = inspect_(chord_2).get_markup()[0]
    assert markup_1 == markup_2
    assert markup_1 is not markup_2



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHead_tweak_01():

    chord = Chord([0, 2, 10], (1, 4))

    chord.note_heads[0].tweak.color = 'red'
    chord.note_heads[0].tweak.thickness = 2

    chord.note_heads[1].tweak.color = 'red'
    chord.note_heads[1].tweak.thickness = 2

    chord.note_heads[2].tweak.color = 'blue'

    tweak_reservoir_1 = chord.note_heads[0].tweak
    tweak_reservoir_2 = chord.note_heads[1].tweak
    tweak_reservoir_3 = chord.note_heads[2].tweak

    assert tweak_reservoir_1 == tweak_reservoir_1
    assert tweak_reservoir_1 == tweak_reservoir_2
    assert tweak_reservoir_1 != tweak_reservoir_3
    assert tweak_reservoir_2 == tweak_reservoir_1
    assert tweak_reservoir_2 == tweak_reservoir_2
    assert tweak_reservoir_2 != tweak_reservoir_3
    assert tweak_reservoir_3 != tweak_reservoir_1
    assert tweak_reservoir_3 != tweak_reservoir_2
    assert tweak_reservoir_3 == tweak_reservoir_3



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__misc__default_duration_01():

    target = Container(scoretools.make_notes([0],
        [(1, 4), (1, 2), (1, 2), (1, 8), (1, 8), (3, 16), (3, 16)]))
    attach(Multiplier(5, 17), target[-2])
    attach(Multiplier(5, 17), target[-1])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c'4
            c'2
            c'2
            c'8
            c'8
            c'8. * 5/17
            c'8. * 5/17
        }
        '''
        )

    string = r'''{ c' c'2 c' c'8 c' c'8. * 5/17 c' }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and target is not result
